# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
from pybpodapi.bpod.hardware.events.base_eventname import BaseEventName

logger = logging.getLogger(__name__)


class EventName(BaseEventName):
    """
    Input event codes
    These values must be set according to Bpod firmware specification.
    """

    ##########################
    # Serial 1
    ##########################

    #: Serial1_1
    Serial1_1 = "Serial1_1"

    #: Serial1_2
    Serial1_2 = "Serial1_2"

    #: Serial1_3
    Serial1_3 = "Serial1_3"

    #: Serial1_4
    Serial1_4 = "Serial1_4"

    #: Serial1_5
    Serial1_5 = "Serial1_5"

    #: Serial1_6
    Serial1_6 = "Serial1_6"

    #: Serial1_7
    Serial1_7 = "Serial1_7"

    #: Serial1_8
    Serial1_8 = "Serial1_8"

    #: Serial1_9
    Serial1_9 = "Serial1_9"

    #: Serial1_10
    Serial1_10 = "Serial1_10"

    #: Serial1_11
    Serial1_11 = "Serial1_11"

    #: Serial1_12
    Serial1_12 = "Serial1_12"

    #: Serial1_13
    Serial1_13 = "Serial1_13"

    #: Serial1_14
    Serial1_14 = "Serial1_14"

    #: Serial1_15
    Serial1_15 = "Serial1_15"

    #: Serial1_16
    Serial1_16 = "Serial1_16"

    #: Serial1_17
    Serial1_17 = "Serial1_17"

    #: Serial1_18
    Serial1_18 = "Serial1_18"

    #: Serial1_19
    Serial1_19 = "Serial1_19"

    #: Serial1_20
    Serial1_20 = "Serial1_20"

    #: Serial1_21
    Serial1_21 = "Serial1_21"

    #: Serial1_22
    Serial1_22 = "Serial1_22"

    #: Serial1_23
    Serial1_23 = "Serial1_23"

    #: Serial1_24
    Serial1_24 = "Serial1_24"

    #: Serial1_25
    Serial1_25 = "Serial1_25"

    #: Serial1_26
    Serial1_26 = "Serial1_26"

    #: Serial1_27
    Serial1_27 = "Serial1_27"

    #: Serial1_28
    Serial1_28 = "Serial1_28"

    #: Serial1_29
    Serial1_29 = "Serial1_29"

    #: Serial1_30
    Serial1_30 = "Serial1_30"

    #: Serial1_31
    Serial1_31 = "Serial1_31"

    #: Serial1_32
    Serial1_32 = "Serial1_32"

    #: Serial1_33
    Serial1_33 = "Serial1_33"

    #: Serial1_34
    Serial1_34 = "Serial1_34"

    #: Serial1_35
    Serial1_35 = "Serial1_35"

    #: Serial1_36
    Serial1_36 = "Serial1_36"

    #: Serial1_37
    Serial1_37 = "Serial1_37"

    #: Serial1_38
    Serial1_38 = "Serial1_38"

    #: Serial1_39
    Serial1_39 = "Serial1_39"

    #: Serial1_40
    Serial1_40 = "Serial1_40"

    #: Serial1_41
    Serial1_41 = "Serial1_41"

    #: Serial1_42
    Serial1_42 = "Serial1_42"

    #: Serial1_43
    Serial1_43 = "Serial1_43"

    #: Serial1_44
    Serial1_44 = "Serial1_44"

    #: Serial1_45
    Serial1_45 = "Serial1_45"

    #: Serial1_46
    Serial1_46 = "Serial1_46"

    #: Serial1_47
    Serial1_47 = "Serial1_47"

    #: Serial1_48
    Serial1_48 = "Serial1_48"

    #: Serial1_49
    Serial1_49 = "Serial1_49"

    #: Serial1_50
    Serial1_50 = "Serial1_50"

    #: Serial1_51
    Serial1_51 = "Serial1_51"

    #: Serial1_52
    Serial1_52 = "Serial1_52"

    #: Serial1_53
    Serial1_53 = "Serial1_53"

    #: Serial1_54
    Serial1_54 = "Serial1_54"

    #: Serial1_55
    Serial1_55 = "Serial1_55"

    #: Serial1_56
    Serial1_56 = "Serial1_56"

    #: Serial1_57
    Serial1_57 = "Serial1_57"

    #: Serial1_58
    Serial1_58 = "Serial1_58"

    #: Serial1_59
    Serial1_59 = "Serial1_59"

    #: Serial1_60
    Serial1_60 = "Serial1_60"

    ##########################
    # Serial 2
    ##########################

    #: Serial2_1
    Serial2_1 = "Serial2_1"

    #: Serial2_2
    Serial2_2 = "Serial2_2"

    #: Serial2_3
    Serial2_3 = "Serial2_3"

    #: Serial2_4
    Serial2_4 = "Serial2_4"

    #: Serial2_5
    Serial2_5 = "Serial2_5"

    #: Serial2_6
    Serial2_6 = "Serial2_6"

    #: Serial2_7
    Serial2_7 = "Serial2_7"

    #: Serial2_8
    Serial2_8 = "Serial2_8"

    #: Serial2_9
    Serial2_9 = "Serial2_9"

    #: Serial2_10
    Serial2_10 = "Serial2_10"

    #: Serial2_11
    Serial2_11 = "Serial2_11"

    #: Serial2_12
    Serial2_12 = "Serial2_12"

    #: Serial2_13
    Serial2_13 = "Serial2_13"

    #: Serial2_14
    Serial2_14 = "Serial2_14"

    #: Serial2_15
    Serial2_15 = "Serial2_15"

    #: Serial2_16
    Serial2_16 = "Serial2_16"

    #: Serial2_17
    Serial2_17 = "Serial2_17"

    #: Serial2_18
    Serial2_18 = "Serial2_18"

    #: Serial2_19
    Serial2_19 = "Serial2_19"

    #: Serial2_20
    Serial2_20 = "Serial2_20"

    #: Serial2_21
    Serial2_21 = "Serial2_21"

    #: Serial2_22
    Serial2_22 = "Serial2_22"

    #: Serial2_23
    Serial2_23 = "Serial2_23"

    #: Serial2_24
    Serial2_24 = "Serial2_24"

    #: Serial2_25
    Serial2_25 = "Serial2_25"

    #: Serial2_26
    Serial2_26 = "Serial2_26"

    #: Serial2_27
    Serial2_27 = "Serial2_27"

    #: Serial2_28
    Serial2_28 = "Serial2_28"

    #: Serial2_29
    Serial2_29 = "Serial2_29"

    #: Serial2_30
    Serial2_30 = "Serial2_30"

    #: Serial2_31
    Serial2_31 = "Serial2_31"

    #: Serial2_32
    Serial2_32 = "Serial2_32"

    #: Serial2_33
    Serial2_33 = "Serial2_33"

    #: Serial2_34
    Serial2_34 = "Serial2_34"

    #: Serial2_35
    Serial2_35 = "Serial2_35"

    #: Serial2_36
    Serial2_36 = "Serial2_36"

    #: Serial2_37
    Serial2_37 = "Serial2_37"

    #: Serial2_38
    Serial2_38 = "Serial2_38"

    #: Serial2_39
    Serial2_39 = "Serial2_39"

    #: Serial2_40
    Serial2_40 = "Serial2_40"

    #: Serial2_41
    Serial2_41 = "Serial2_41"

    #: Serial2_42
    Serial2_42 = "Serial2_42"

    #: Serial2_43
    Serial2_43 = "Serial2_43"

    #: Serial2_44
    Serial2_44 = "Serial2_44"

    #: Serial2_45
    Serial2_45 = "Serial2_45"

    #: Serial2_46
    Serial2_46 = "Serial2_46"

    #: Serial2_47
    Serial2_47 = "Serial2_47"

    #: Serial2_48
    Serial2_48 = "Serial2_48"

    #: Serial2_49
    Serial2_49 = "Serial2_49"

    #: Serial2_50
    Serial2_50 = "Serial2_50"

    #: Serial2_51
    Serial2_51 = "Serial2_51"

    #: Serial2_52
    Serial2_52 = "Serial2_52"

    #: Serial2_53
    Serial2_53 = "Serial2_53"

    #: Serial2_54
    Serial2_54 = "Serial2_54"

    #: Serial2_55
    Serial2_55 = "Serial2_55"

    #: Serial2_56
    Serial2_56 = "Serial2_56"

    #: Serial2_57
    Serial2_57 = "Serial2_57"

    #: Serial2_58
    Serial2_58 = "Serial2_58"

    #: Serial2_59
    Serial2_59 = "Serial2_59"

    #: Serial2_60
    Serial2_60 = "Serial2_60"

    ##########################
    # Serial 3
    ##########################

    #: Serial3_1
    Serial3_1 = "Serial3_1"

    #: Serial3_2
    Serial3_2 = "Serial3_2"

    #: Serial3_3
    Serial3_3 = "Serial3_3"

    #: Serial3_4
    Serial3_4 = "Serial3_4"

    #: Serial3_5
    Serial3_5 = "Serial3_5"

    #: Serial3_6
    Serial3_6 = "Serial3_6"

    #: Serial3_7
    Serial3_7 = "Serial3_7"

    #: Serial3_8
    Serial3_8 = "Serial3_8"

    #: Serial3_9
    Serial3_9 = "Serial3_9"

    #: Serial3_10
    Serial3_10 = "Serial3_10"

    #: Serial3_11
    Serial3_11 = "Serial3_11"

    #: Serial3_12
    Serial3_12 = "Serial3_12"

    #: Serial3_13
    Serial3_13 = "Serial3_13"

    #: Serial3_14
    Serial3_14 = "Serial3_14"

    #: Serial3_15
    Serial3_15 = "Serial3_15"

    #: Serial3_16
    Serial3_16 = "Serial3_16"

    #: Serial3_17
    Serial3_17 = "Serial3_17"

    #: Serial3_18
    Serial3_18 = "Serial3_18"

    #: Serial3_19
    Serial3_19 = "Serial3_19"

    #: Serial3_20
    Serial3_20 = "Serial3_20"

    #: Serial3_21
    Serial3_21 = "Serial3_21"

    #: Serial3_22
    Serial3_22 = "Serial3_22"

    #: Serial3_23
    Serial3_23 = "Serial3_23"

    #: Serial3_24
    Serial3_24 = "Serial3_24"

    #: Serial3_25
    Serial3_25 = "Serial3_25"

    #: Serial3_26
    Serial3_26 = "Serial3_26"

    #: Serial3_27
    Serial3_27 = "Serial3_27"

    #: Serial3_28
    Serial3_28 = "Serial3_28"

    #: Serial3_29
    Serial3_29 = "Serial3_29"

    #: Serial3_30
    Serial3_30 = "Serial3_30"

    #: Serial3_31
    Serial3_31 = "Serial3_31"

    #: Serial3_32
    Serial3_32 = "Serial3_32"

    #: Serial3_33
    Serial3_33 = "Serial3_33"

    #: Serial3_34
    Serial3_34 = "Serial3_34"

    #: Serial3_35
    Serial3_35 = "Serial3_35"

    #: Serial3_36
    Serial3_36 = "Serial3_36"

    #: Serial3_37
    Serial3_37 = "Serial3_37"

    #: Serial3_38
    Serial3_38 = "Serial3_38"

    #: Serial3_39
    Serial3_39 = "Serial3_39"

    #: Serial3_40
    Serial3_40 = "Serial3_40"

    #: Serial3_41
    Serial3_41 = "Serial3_41"

    #: Serial3_42
    Serial3_42 = "Serial3_42"

    #: Serial3_43
    Serial3_43 = "Serial3_43"

    #: Serial3_44
    Serial3_44 = "Serial3_44"

    #: Serial3_45
    Serial3_45 = "Serial3_45"

    #: Serial3_46
    Serial3_46 = "Serial3_46"

    #: Serial3_47
    Serial3_47 = "Serial3_47"

    #: Serial3_48
    Serial3_48 = "Serial3_48"

    #: Serial3_49
    Serial3_49 = "Serial3_49"

    #: Serial3_50
    Serial3_50 = "Serial3_50"

    #: Serial3_51
    Serial3_51 = "Serial3_51"

    #: Serial3_52
    Serial3_52 = "Serial3_52"

    #: Serial3_53
    Serial3_53 = "Serial3_53"

    #: Serial3_54
    Serial3_54 = "Serial3_54"

    #: Serial3_55
    Serial3_55 = "Serial3_55"

    #: Serial3_56
    Serial3_56 = "Serial3_56"

    #: Serial3_57
    Serial3_57 = "Serial3_57"

    #: Serial3_58
    Serial3_58 = "Serial3_58"

    #: Serial3_59
    Serial3_59 = "Serial3_59"

    #: Serial3_60
    Serial3_60 = "Serial3_60"

    ##########################
    # Soft Codes
    ##########################

    #: SoftCode1
    SoftCode1 = "SoftCode1"

    #: SoftCode2
    SoftCode2 = "SoftCode2"

    #: SoftCode3
    SoftCode3 = "SoftCode3"

    #: SoftCode4
    SoftCode4 = "SoftCode4"

    #: SoftCode5
    SoftCode5 = "SoftCode5"

    #: SoftCode6
    SoftCode6 = "SoftCode6"

    #: SoftCode7
    SoftCode7 = "SoftCode7"

    #: SoftCode8
    SoftCode8 = "SoftCode8"

    #: SoftCode9
    SoftCode9 = "SoftCode9"

    #: SoftCode10
    SoftCode10 = "SoftCode10"

    #: SoftCode11
    SoftCode11 = "SoftCode11"

    #: SoftCode12
    SoftCode12 = "SoftCode12"

    #: SoftCode13
    SoftCode13 = "SoftCode13"

    #: SoftCode14
    SoftCode14 = "SoftCode14"

    #: SoftCode15
    SoftCode15 = "SoftCode15"

    #: SoftCode16
    SoftCode16 = "SoftCode16"

    #: SoftCode17
    SoftCode17 = "SoftCode17"

    #: SoftCode18
    SoftCode18 = "SoftCode18"

    #: SoftCode19
    SoftCode19 = "SoftCode19"

    #: SoftCode20
    SoftCode20 = "SoftCode20"

    #: SoftCode21
    SoftCode21 = "SoftCode21"

    #: SoftCode22
    SoftCode22 = "SoftCode22"

    #: SoftCode23
    SoftCode23 = "SoftCode23"

    #: SoftCode24
    SoftCode24 = "SoftCode24"

    #: SoftCode25
    SoftCode25 = "SoftCode25"

    #: SoftCode26
    SoftCode26 = "SoftCode26"

    #: SoftCode27
    SoftCode27 = "SoftCode27"

    #: SoftCode28
    SoftCode28 = "SoftCode28"

    #: SoftCode29
    SoftCode29 = "SoftCode29"

    #: SoftCode30
    SoftCode30 = "SoftCode30"

    #: SoftCode31
    SoftCode31 = "SoftCode31"

    #: SoftCode32
    SoftCode32 = "SoftCode32"

    #: SoftCode33
    SoftCode33 = "SoftCode33"

    #: SoftCode34
    SoftCode34 = "SoftCode34"

    #: SoftCode35
    SoftCode35 = "SoftCode35"

    #: SoftCode36
    SoftCode36 = "SoftCode36"

    #: SoftCode37
    SoftCode37 = "SoftCode37"

    #: SoftCode38
    SoftCode38 = "SoftCode38"

    #: SoftCode39
    SoftCode39 = "SoftCode39"

    #: SoftCode40
    SoftCode40 = "SoftCode40"

    #: SoftCode41
    SoftCode41 = "SoftCode41"

    #: SoftCode42
    SoftCode42 = "SoftCode42"

    #: SoftCode43
    SoftCode43 = "SoftCode43"

    #: SoftCode44
    SoftCode44 = "SoftCode44"

    #: SoftCode45
    SoftCode45 = "SoftCode45"

    #: SoftCode46
    SoftCode46 = "SoftCode46"

    #: SoftCode47
    SoftCode47 = "SoftCode47"

    #: SoftCode48
    SoftCode48 = "SoftCode48"

    #: SoftCode49
    SoftCode49 = "SoftCode49"

    #: SoftCode50
    SoftCode50 = "SoftCode50"

    #: SoftCode51
    SoftCode51 = "SoftCode51"

    #: SoftCode52
    SoftCode52 = "SoftCode52"

    #: SoftCode53
    SoftCode53 = "SoftCode53"

    #: SoftCode54
    SoftCode54 = "SoftCode54"

    #: SoftCode55
    SoftCode55 = "SoftCode55"

    #: SoftCode56
    SoftCode56 = "SoftCode56"

    #: SoftCode57
    SoftCode57 = "SoftCode57"

    #: SoftCode58
    SoftCode58 = "SoftCode58"

    #: SoftCode59
    SoftCode59 = "SoftCode59"

    #: SoftCode60
    SoftCode60 = "SoftCode60"

    #: BNC1In
    BNC1High = "BNC1High"

    #: BNC1Out
    BNC1Low = "BNC1Low"

    #: BNC2In
    BNC2High = "BNC2High"

    #: BNC2Out
    BNC2Low = "BNC2Low"

    #: Wire1In
    Wire1High = "Wire1High"

    #: Wire1Out
    Wire1Low = "Wire1Low"

    #: Wire2In
    Wire2High = "Wire2High"

    #: Wire2Out
    Wire2Low = "Wire2Low"

    #: Input port 1
    Port1In = "Port1In"

    #: Output port 1
    Port1Out = "Port1Out"

    #: Input port 2
    Port2In = "Port2In"

    #: Output port 2
    Port2Out = "Port2Out"

    #: Input port 3
    Port3In = "Port3In"

    #: Output port 3
    Port3Out = "Port3Out"

    #: Input port 4
    Port4In = "Port4In"

    #: Output port 4
    Port4Out = "Port4Out"

    #: Input port 5
    Port5In = "Port5In"

    #: Output port 5
    Port5Out = "Port5Out"

    #: Input port 6
    Port6In = "Port6In"

    #: Output port 6
    Port6Out = "Port6Out"

    #: Input port 7
    Port7In = "Port7In"

    #: Output port 7
    Port7Out = "Port7Out"

    #: Input port 8
    Port8In = "Port8In"

    #: Output port 8
    Port8Out = "Port8Out"

    #: GlobalTimer1_Start
    GlobalTimer1_Start = "GlobalTimer1_Start"

    #: GlobalTimer2_Start
    GlobalTimer2_Start = "GlobalTimer2_Start"

    #: GlobalTimer3_Start
    GlobalTimer3_Start = "GlobalTimer3_Start"

    #: GlobalTimer4_Start
    GlobalTimer4_Start = "GlobalTimer4_Start"

    #: GlobalTimer5_Start
    GlobalTimer5_Start = "GlobalTimer5_Start"

    #: GlobalTimer1_End
    GlobalTimer1_End = "GlobalTimer1_End"

    #: GlobalTimer2_End
    GlobalTimer2_End = "GlobalTimer2_End"

    #: GlobalTimer3_End
    GlobalTimer3_End = "GlobalTimer3_End"

    #: GlobalTimer4_End
    GlobalTimer4_End = "GlobalTimer4_End"

    #: GlobalTimer5_End
    GlobalTimer5_End = "GlobalTimer5_End"

    #: GlobalCounter1_End
    GlobalCounter1_End = "GlobalCounter1_End"

    #: GlobalCounter2_End
    GlobalCounter2_End = "GlobalCounter2_End"

    #: GlobalCounter3_End
    GlobalCounter3_End = "GlobalCounter3_End"

    #: GlobalCounter4_End
    GlobalCounter4_End = "GlobalCounter4_End"

    #: GlobalCounter5_End
    GlobalCounter5_End = "GlobalCounter5_End"

    #: Condition1
    Condition1 = "Condition1"

    #: Condition2
    Condition2 = "Condition2"

    #: Condition3
    Condition3 = "Condition3"

    #: Condition4
    Condition4 = "Condition4"

    #: Condition5
    Condition5 = "Condition5"

    #: Condition6
    Condition6 = "Condition6"

    #: Condition7
    Condition7 = "Condition7"

    #: Condition8
    Condition8 = "Condition8"

    #: Condition9
    Condition9 = "Condition9"

    #: Condition10
    Condition10 = "Condition10"

    #: Condition11
    Condition11 = "Condition11"

    #: Condition12
    Condition12 = "Condition12"

    #: Condition13
    Condition13 = "Condition13"

    #: Condition14
    Condition14 = "Condition14"

    #: Condition15
    Condition15 = "Condition15"

    #: Condition16
    Condition16 = "Condition16"

    #: Tup
    Tup = "Tup"



# # !/usr/bin/python3
# # -*- coding: utf-8 -*-

import logging, os

SETTINGS_PRIORITY = 100



# THESE SETTINGS ARE NEEDED FOR PYSETTINGS
APP_LOG_FILENAME = 'app.log'
APP_LOG_HANDLER_CONSOLE_LEVEL = logging.WARNING
APP_LOG_HANDLER_FILE_LEVEL 	  = logging.WARNING

CONTROL_EVENTS_GRAPH_DEFAULT_SCALE 	= 100
BOARD_LOG_WINDOW_REFRESH_RATE 		= 1000

USE_MULTIPROCESSING = True

PYFORMS_MAINWINDOW_MARGIN 		= 0
PYFORMS_STYLESHEET 				= ''
PYFORMS_STYLESHEET_DARWIN 		= ''
PYFORMS_SILENT_PLUGINS_FINDER 	= True


#PYFORMS_STYLESHEET = os.path.join(os.path.dirname(__file__), 'resources', 'css', 'default.css')

PYFORMS_MATPLOTLIB_ENABLED 	= True
PYFORMS_WEB_ENABLED 		= True
PYFORMS_GL_ENABLED 			= True
PYFORMS_VISVIS_ENABLED 		= False

GENERIC_EDITOR_PLUGINS_PATH = None
GENERIC_EDITOR_PLUGINS_LIST = [
    'pybpodgui_plugin',
    'pybpodgui_plugin_timeline',
    'pybpodgui_plugin_trial_timeline',
    'pybpod_alyx_plugin',
    'pybpodgui_plugin_session_history',
#	'pge_welcome_plugin',
]

#WELCOME_PLUGIN_URL = 'http://pybpod.readthedocs.io'


############ BPODGUI PLUGIN SETTINGS ############

#DEFAULT_PROJECT_PATH = '/home/ricardo/bitbucket/pybpod/pybpod-gui-plugin/projects/Untitled project 1'

BOARD_LOG_WINDOW_REFRESH_RATE  = 2000
SESSIONLOG_PLUGIN_REFRESH_RATE = 1000
TIMELINE_PLUGIN_REFRESH_RATE   = 1000

PYBOARD_COMMUNICATION_THREAD_REFRESH_TIME  = 2 # timer for thread look for events (seconds)
PYBOARD_COMMUNICATION_PROCESS_REFRESH_TIME = 2 # timer for process look for events (seconds)
PYBOARD_COMMUNICATION_PROCESS_TIME_2_LIVE  = 0 # wait before killing process (seconds)

GENERIC_EDITOR_TITLE = 'PyBpod'


PYBPOD_REPOSITORIES_TXT_LIST = 'repositories.yml'


from threading import Thread, Event
from queue import Queue, Empty
from sys import platform

if platform in ["linux", "linux2", "darwin"]:
    import fcntl
    import os


class NonBlockingStreamReader:
    def __init__(self, stream):
        """
        stream: the stream to read from.
                Usually a process' stdout or stderr.
        """
        self._s = stream
        self._q = Queue()

        if platform in ["linux", "linux2", "darwin"]:
            fd = stream.fileno()
            fl = fcntl.fcntl(fd, fcntl.F_GETFL)
            fcntl.fcntl(fd, fcntl.F_SETFL, fl | os.O_NONBLOCK)

        class PopulateQueue(Thread):
            def __init__(self, stream, queue):
                Thread.__init__(self)
                self.daemon = True
                self.stream = stream
                self.queue = queue
                self.event = Event()

            def run(self):
                while True:
                    if self.event.is_set():
                        break
                    line = self.stream.readline()
                    if line:
                        self.queue.put(line)
                        lines = None
                    self.event.wait(0.01)

        self._t = PopulateQueue(self._s, self._q)
        self._t.daemon = True
        self._t.start()  # start collecting lines from the stream

    def readline(self, timeout=None):
        try:
            return self._q.get(block=timeout is not None, timeout=timeout)
        except Empty:
            return None

    def close(self):
        self._t.event.set()

    def is_alive(self):
        return self._t.is_alive()


class UnexpectedEndOfStream(Exception):
    pass



# !/usr/bin/python3
# -*- coding: utf-8 -*-

PYBPOD_API_LOG_LEVEL = None
PYBPOD_API_LOG_FILE = "pybpod-api.log"

PYBPOD_API_MODULES = [
    # 'pybpod_rotaryencoder_module'
]

# stream the session file to the stdin (terminal)
PYBPOD_API_STREAM2STDOUT = True
# accept commands from the stdin
PYBPOD_API_ACCEPT_STDIN = False

# SUPPORTED BPOD FIRMWARE VERSION
# TARGET_BPOD_FIRMWARE_VERSION = "9"  # 0.7.5
# TARGET_BPOD_FIRMWARE_VERSION = "13" # 0.7.9
# TARGET_BPOD_FIRMWARE_VERSION = "15" # 0.8
# TARGET_BPOD_FIRMWARE_VERSION = "17" # 0.9
TARGET_BPOD_FIRMWARE_VERSION = "22"

PYBPOD_SERIAL_PORT = None
PYBPOD_NET_PORT = None
PYBPOD_BAUDRATE = 1312500
PYBPOD_SYNC_CHANNEL = 255
PYBPOD_SYNC_MODE = 1


BPOD_BNC_PORTS_ENABLED = [True, True]
BPOD_WIRED_PORTS_ENABLED = [True, True]
BPOD_BEHAVIOR_PORTS_ENABLED = [True, True, True, True, True, True, True, True]

PYBPOD_SERIAL_PORT = ""

PYBPOD_PROTOCOL = ""
PYBPOD_CREATOR = ""
PYBPOD_PROJECT = ""
PYBPOD_EXPERIMENT = ""
PYBPOD_BOARD = ""
PYBPOD_SETUP = ""
PYBPOD_SESSION = ""
PYBPOD_SESSION_PATH = ""
PYBPOD_BPODGUI_VERSION = ""
PYBPOD_SUBJECTS = []



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
import uuid
import os
from pybpodgui_api.models.setup import Setup
from pybpodgui_api.utils.generate_name import generate_name

logger = logging.getLogger(__name__)


class ExperimentBase(object):
    """
    Experiment entity details.
    Each experiment should have a name, a list of setups, a project belonging to, a path and a task.
    """

    def __init__(self, project):
        """
        :ivar Project project: project object reference
        """
        self.uuid4 = uuid.uuid4()

        self.name = generate_name([x.name for x in project.experiments], "experiment")
        self._setups = []
        self.project = project

        self.project += self

    ##########################################################################
    ####### PROPERTIES #######################################################
    ##########################################################################

    @property
    def name(self):
        """
        Get and set the experiment name

        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    @property
    def task(self):
        """
        Get and set the experiment task

        :rtype: Task
        """
        return self._task

    @task.setter
    def task(self, value):
        if isinstance(value, str):
            value = self.project.find_task(value)

        self._task = value

        for setup in self.setups:
            setup.task = value

    @property
    def project(self):
        """
        Get and set the experiment project

        :rtype: Project
        """
        return self._project

    @project.setter
    def project(self, value):
        self._project = value

    @property
    def setups(self):
        """
        Get the experiment setups

        :rtype: list(Setup)
        """
        return self._setups

    @property
    def path(self):
        """
        Get and set the experiment files path

        :rtype: str
        """
        if self.project.path is None:
            return None

        return os.path.join(self.project.path, 'experiments', self.name)

    ##########################################################################
    ####### FUNCTIONS ########################################################
    ##########################################################################

    def remove(self):
        """
        Remove experiment

        """
        pass

    def create_setup(self):
        """
        Create new instance of setup

        :rtype: Setup
        """
        return Setup(self)

    def __add__(self, obj):
        if isinstance(obj, Setup):
            self._setups.append(obj)
        return self

    def __sub__(self, obj):
        if isinstance(obj, Setup):
            self._setups.remove(obj)
        return self

    def __unicode__(self):
        return self.name

    def __str__(self):
        return self.__unicode__()



# !/usr/bin/python3
# -*- coding: utf-8 -*-

from pybpodapi.com.messaging.base_message import BaseMessage
from pybpodapi.utils import date_parser


class StderrMessage(BaseMessage):
    """
    Stderr message from the server process

    .. seealso::

        :py:class:`pybpodgui_plugin.com.messaging.base_message.BoardMessage`

    """

    MESSAGE_TYPE_ALIAS = "stderr"
    MESSAGE_COLOR = (255, 0, 0)

    def __init__(self, content, host_timestamp=None):
        super(StderrMessage, self).__init__(str(content), host_timestamp)

    def __str__(self):
        return "host-time:{0} pc-time:{1} {2}".format(
            self.host_timestamp if self.host_timestamp is not None else "",
            self.pc_timestamp.strftime("%Y%m%d%H%M%S") if self.pc_timestamp else "",
            self.content,
        )

    def tolist(self):
        return [
            self.MESSAGE_TYPE_ALIAS,
            str(self.pc_timestamp),
            self.host_timestamp,
            self.content,
            None,
            None,
        ]

    @classmethod
    def fromlist(cls, row):
        """
        Returns True if the typestr represents the class
        """
        obj = cls(row[3], float(row[2]) if row[2] else None)
        obj.pc_timestamp = date_parser.parse(row[1])

        return obj



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

logger = logging.getLogger(__name__)


class SendMessageHeader(object):
    """
    Define names for message headers sent to the Bpod device.

    The message header is the first byte (character) on a message sent.
    """

    #: Request initialization handshake
    HANDSHAKE = "6"

    #: Request firmware build number
    FIRMWARE_VERSION = "F"

    #: Reset session clock
    RESET_CLOCK = "*"

    #: Pause ongoing trial (We recommend using computer-side pauses between trials, to keep data uniform)
    PAUSE_TRIAL = "$"

    #: Return timestamp transmission scheme
    GET_TIMESTAMP_TRANSMISSION = "G"

    #: Request hardware configuration
    HARDWARE_DESCRIPTION = "H"

    #: Request enable input ports
    ENABLE_PORTS = "E"

    #: Set sync channel and sync mode
    SYNC_CHANNEL_MODE = "K"

    #: Send new compressed state matrix
    NEW_STATE_MATRIX = "C"

    #: Request to run state matrix now
    RUN_STATE_MACHINE = "R"

    #: Load serial message
    LOAD_SERIAL_MESSAGE = "L"

    #: Reset serial messages to equivalent byte codes (i.e. message# 4 = one byte, 0x4)
    RESET_SERIAL_MESSAGES = ">"

    #: Override digital hardware state
    OVERRIDE_DIGITAL_HW_STATE = "O"

    #: Send byte to hardware serial channel 1-3
    SEND_TO_HW_SERIAL = "U"

    #: Request end of connection now
    DISCONNECT = "Z"

    #: Get the modules connected to bpod
    GET_MODULES = "M"

    #: Set module relay
    SET_MODULE_RELAY = "J"

    #: Write to the module
    WRITE_TO_MODULE = "T"

    #: Echo soft code
    ECHO_SOFTCODE = "S"

    #: Manual override: execute virtual event
    MANUAL_OVERRIDE_EXEC_EVENT = "V"

    #: Trigger soft code
    TRIGGER_SOFTCODE = "~"

    #: Exit state matrix and return data
    EXIT_AND_RETURN = "X"



# !/usr/bin/python3
# -*- coding: utf-8 -*-

from confapp import conf

from pybpodgui_plugin.models.setup.setup_uibusy import SetupUIBusy

Setup = type(
    'Setup',
    tuple(conf.GENERIC_EDITOR_PLUGINS_FINDER.find_class('models.setup.Setup') + [SetupUIBusy]),
    {}
)



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

from pybpodgui_plugin.models.experiment.experiment_treenode import ExperimentTreeNode

logger = logging.getLogger(__name__)


class ExperimentDockWindow(ExperimentTreeNode):
    """
    Dock window settings.
    Define here behaviors associated with experiment details section.

    **Properties**

        mainwindow
            Returns project main window.

    **Methods**

    """

    MARKED_FOR_REMOVAL = False  # This flag will be activated while removing experiment and its setups

    def show(self):
        """
        Select this window as the main window on the details section.
        Also reload tasks list on combo box.
        """
        self.mainwindow.details.value = self
        # self.reload_tasks(current_selected_task=self.task)
        super(ExperimentDockWindow, self).show()

    def focus_name(self):
        """
        Sets interface focus on the board name text field
        """
        self._name.form.lineEdit.setFocus()

    def remove(self):
        """
        Prompts user to confirm experiment removal.

        .. seealso::
            This method extends experiment tree node :py:meth:`pybpodgui_plugin.models.experiment.experiment_treenode.ExperimentTreeNode.remove`.

        """
        reply = self.question('Experiment "{0}" and all setups will be deleted. Are you sure?'.format(self.name), "Warning")

        if reply == 'yes':
            self.MARKED_FOR_REMOVAL = True
            self.mainwindow.details.value = None
            super(ExperimentDockWindow, self).remove()

    @property
    def mainwindow(self):
        return self.project.mainwindow



# !/usr/bin/python
# -*- coding: utf-8 -*-

import os
from AnyQt.QtGui import QIcon, QPixmap
from AnyQt.QtWidgets import QStyle, qApp, QFileDialog, QColorDialog

def path(filename):
	"""	
	:param filename: 
	:return: 
	"""
	return os.path.join(os.path.dirname(__file__), filename)


PYFORMS_ICON_VIDEOPLAYER_PAUSE_PLAY = QIcon()
PYFORMS_ICON_VIDEOPLAYER_PAUSE_PLAY.addPixmap(qApp.style().standardPixmap(QStyle.SP_MediaPlay), mode=QIcon.Normal,
                                              state=QIcon.Off)
PYFORMS_ICON_VIDEOPLAYER_PAUSE_PLAY.addPixmap(qApp.style().standardPixmap(QStyle.SP_MediaPause), mode=QIcon.Normal,
                                              state=QIcon.On)

PYFORMS_ICON_VIDEOPLAYER_DETACH = QIcon()
PYFORMS_ICON_VIDEOPLAYER_DETACH.addPixmap(qApp.style().standardPixmap(QStyle.SP_TitleBarMaxButton), mode=QIcon.Normal )

PYFORMS_ICON_CODEEDITOR_SAVE = QIcon(qApp.style().standardPixmap(QStyle.SP_DialogSaveButton))
PYFORMS_ICON_CODEEDITOR_DISCARD = QIcon(qApp.style().standardPixmap(QStyle.SP_DialogDiscardButton))

PYFORMS_PIXMAP_EVENTTIMELINE_ZOOM_IN = QPixmap(path(os.path.join("Controls", "uipics", "zoom_in.png")))
PYFORMS_PIXMAP_EVENTTIMELINE_ZOOM_OUT = QPixmap(path(os.path.join("Controls", "uipics", "zoom_in.png")))

PYFORMS_ICON_EVENTTIMELINE_IMPORT = QIcon(path(os.path.join("Controls", "uipics", "page_white_get.png")))
PYFORMS_ICON_EVENTTIMELINE_EXPORT = QIcon(path(os.path.join("Controls", "uipics", "page_white_put.png")))
PYFORMS_ICON_EVENTTIMELINE_GRAPH = QIcon(path(os.path.join("Controls", "uipics", "graph.png")))
PYFORMS_ICON_EVENTTIMELINE_TIMELINE = QIcon(path(os.path.join("Controls", "uipics", "timeline.png")))
PYFORMS_ICON_EVENTTIMELINE_REFRESH = QIcon(path(os.path.join("Controls", "uipics", "refresh.png")))
PYFORMS_ICON_EVENTTIMELINE_ADD = QIcon(path(os.path.join("Controls", "uipics", "add.png")))
PYFORMS_ICON_EVENTTIMELINE_REMOVE = QIcon(path(os.path.join("Controls", "uipics", "remove.png")))

PYFORMS_ICON_FILE_OPEN = QIcon()

PYFORMS_MAIN_WINDOW_ICON_PATH = None

PYFORMS_MAINWINDOW_MARGIN = 7

PYFORMS_CONTROL_CODE_EDITOR_DEFAULT_FONT_SIZE = '12'
PYFORMS_CONTROL_EVENTS_GRAPH_DEFAULT_SCALE = 1

PYFORMS_QUALITY_TESTS_PATH = None

PYFORMS_STYLESHEET = None
PYFORMS_STYLESHEET_DARWIN = None
PYFORMS_STYLESHEET_LINUX = None
PYFORMS_STYLESHEET_WINDOWS = None

PYFORMS_CONTROLPLAYER_FONT = 9

# In a normal loading, there may be errors that show up which are not important.
# This happens because plugins_finder will search for classes on plugins which are not present because they are not needed.
# However, if plugin is not loaded at all, this will show all related errors.
# See pyforms_gui.utils.plugins_finder.find_class()
PYFORMS_SILENT_PLUGINS_FINDER = True

PYFORMS_QSCINTILLA_ENABLED 	= True
PYFORMS_MATPLOTLIB_ENABLED 	= True
PYFORMS_WEB_ENABLED 		= True
PYFORMS_GL_ENABLED 			= True
PYFORMS_VISVIS_ENABLED 		= True


PYFORMS_DIALOGS_OPTIONS = None
#PYFORMS_DIALOGS_OPTIONS = QFileDialog.DontUseNativeDialog
PYFORMS_COLORDIALOGS_OPTIONS = QColorDialog.DontUseNativeDialog


# !/usr/bin/python3
# -*- coding: utf-8 -*-

from pybpodgui_plugin.models.session.session_dockwindow import SessionDockWindow


class SessionSignals(SessionDockWindow):
    """
    Session Signals offers a set of useful pyqt signals
    """

    def create_treenode(self, tree):
        super().create_treenode(tree)

        self.project.projects.signal_session_create_treenode.emit(self)



# !/usr/bin/python3
# -*- coding: utf-8 -*-


from pybpodgui_plugin.models.projects.projects_window import ProjectsWindow
from pybpodgui_plugin import utils


class ProjectsTreeNode(ProjectsWindow):

    def create_project(self):
        """
        Invoke project creation and focus GUI on the new tree node.

        .. seealso::
            * Create project: :class:`pybpodgui_plugin.models.projects.projects_window.ProjectsWindow.create_project`.

        :rtype: Project
        """
        project = super().create_project()
        self.tree.setCurrentItem(project.node)
        project.focus_name()
        return project

    def open_project(self, project_path=None):
        """
        Open project. We want to temporarily silent the item_selection_changed_event becasue otherwise, when loading
        the project, entities will show up on the MDI area (and not in the dockwindow).
        Finally, we have to force the project node to change.

        .. seealso::
            * Open project: :class:`pybpodgui_plugin.models.projects.projects_window.ProjectsWindow.open_project`.

        :param str project_path:
        """
        try:
            self.tree.item_selection_changed_event = utils.do_nothing
            project = super().open_project(project_path)
            if project:
                self.tree.item_selection_changed_event = self.__item_sel_changed_evt
                self.tree.setCurrentItem(project.node)
                self.__item_sel_changed_evt()
            return project
        except Exception:
            self.tree.item_selection_changed_event = self.__item_sel_changed_evt
            raise



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
from pybpodgui_plugin.models.subject.subject_treenode import SubjectTreeNode

logger = logging.getLogger(__name__)


class SubjectDockWindow(SubjectTreeNode):

    def post_load(self):
        self.reload_setups()
        super().post_load()

    def show(self):
        self.mainwindow.details.value = self
        super(SubjectDockWindow, self).show()

    def focus_name(self):
        """
        Sets interface focus on the board name text field
        """
        self._name.form.lineEdit.setFocus()

    def remove(self, silent=False):
        """

        Prompts user to confirm subject removal and closes mdi windows associated with this subject.

        .. seealso::
            This method extends subject tree node :py:meth:`pybpodgui_plugin.models.subject.subject_treenode.SubjectTreeNode.remove`.

        """
        reply = None
        if not silent:
            reply = self.question('Subject "{0}" will be deleted. Are you sure?'.format(self.name), 'Warning')
        if silent or (reply is not None and reply == 'yes'):
            if hasattr(self, '_code_editor'):
                self.mainwindow.mdi_area -= self._code_editor
            super(SubjectDockWindow, self).remove(silent)

    def close(self, silent=False):
        self.mainwindow.details.value = None
        super(SubjectDockWindow, self).close(silent)

    @property
    def mainwindow(self):
        return self.project.mainwindow



# !/usr/bin/python3
# -*- coding: utf-8 -*-

__version__ = "1.4.2"

from confapp import conf

conf += 'pybpodgui_plugin_session_history.settings'
conf += 'pybpodgui_plugin_session_history.resources'


import loggingbootstrap

# setup different loggers but output to single file
loggingbootstrap.create_double_logger("pybpodgui_plugin_session_history", conf.APP_LOG_HANDLER_CONSOLE_LEVEL,
									  conf.APP_LOG_FILENAME,
									  conf.APP_LOG_HANDLER_FILE_LEVEL)


# !/usr/bin/python3
# -*- coding: utf-8 -*-

import os
import logging
import pybpodgui_api
from .subject_com import SubjectCom

from sca.formats import json

logger = logging.getLogger(__name__)


class SubjectIO(SubjectCom):
    """

    """

    def __init__(self, project):
        super().__init__(project)

        self.data = None

    ##########################################################################
    ####### FUNCTIONS ########################################################
    ##########################################################################

    def save(self):
        """
        Save subject data on filesystem.

        :ivar str project_path: Project path.
        :return: Dictionary containing the setup info to save.
        :rtype: dict
        """
        if not self.name:
            logger.warning("Skipping subject without name")
            return None
        else:
            if not os.path.exists(self.path):
                os.makedirs(self.path)

            if self.data:
                data = self.data
            else:
                data = json.scadict(
                    uuid4_id=self.uuid4,
                    software='PyBpod GUI API v'+str(pybpodgui_api.__version__),
                    def_url='http://pybpod.readthedocs.org',
                    def_text='This file contains information about a subject used on PyBpod GUI.'
                )

            data['setup'] = str(self.setup.uuid4 if self.setup else None)

            config_path = os.path.join(self.path, self.name+'.json')
            with open(config_path, 'w') as fstream:
                json.dump(data, fstream)

    def toJSON(self):
        data = json.scadict(
                    uuid4_id=self.uuid4,
                    software='PyBpod GUI API v'+str(pybpodgui_api.__version__),
                    def_url='http://pybpod.readthedocs.org',
                    def_text='This file contains information about a subject used on PyBpod GUI.',
                )
        data['name'] = self.name
        data['setup'] = str(self.setup.uuid4 if self.setup else None)
        data['uuid4'] = self.uuid4

        return json.dumps(data)

    def load(self, path):
        """
        Load subject data from filesystem

        :ivar str subject_path: Path of the subject
        :ivar dict data: data object that contains all subject info
        """
        self.name = os.path.basename(path)

        try:
            with open(os.path.join(self.path, self.name+'.json'), 'r') as stream:
                self.data = data = json.load(stream)
            self.uuid4 = data.uuid4 if data.uuid4 else self.uuid4

            self._setup_id = self.data.get('setup', None)
        except:
            raise Exception('There was an error loading the configuration file for the subject [{name}]. File not found.'.format(name=self.name))

    def post_load(self):
        self.setup = self.project.find_setup_by_id(self._setup_id) if hasattr(self, "_setup_id") else None

    def collect_data(self, data):
        data.update({'name': self.name})
        data.update({'setup': str(self.setup.uuid4 if self.setup else None)})
        return data



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

from confapp import conf

from AnyQt.QtGui import QIcon

from pybpodgui_plugin.models.subject.subject_dockwindow import SubjectDockWindow

logger = logging.getLogger(__name__)


class SubjectUIBusy(SubjectDockWindow):
    """
    Extends subject with UI refreshment logic.

    """

    def __init__(self, project):
        super(SubjectUIBusy, self).__init__(project)
        self.__running_icon = QIcon(conf.PLAY_SMALL_ICON)

    def pause_trial(self):
        setup = self._setups.value
        if setup:
            if self._pause_btn.checked:
                setup.pause_trial()
                self._pause_btn.label = 'Resume'
            else:
                setup.resume_trial()
                self._pause_btn.label = "Pause"

    # This must switch between start and stop
    def update_ui(self, sessionrunning=False):
        if sessionrunning:
            self._run.checked = True
            self._run.label = 'Stop'
            self._kill_task_btn.enabled = True
            self._stoptrial_btn.enabled = True
            self._pause_btn.enabled = True
            self._detached.enabled = False
            for sess in self._sessions:
                if sess.running:
                    self.node.setIcon(0, self.__running_icon)
        else:
            self._run.checked = False
            self._run.label = 'Run'
            self._kill_task_btn.enabled = False
            self._detached.enabled = True
            self._stoptrial_btn.enabled = False
            self._pause_btn.enabled = False
            self._pause_btn.checked = False
            self.node.setIcon(0, QIcon(conf.SUBJECT_SMALL_ICON))



from sca import format_flags as flags
from dateutil import parser
import csv as csv, uuid, datetime, hashlib, os, sys
from sca.utils import hash_utils


csv.field_size_limit(10000000)

CSV_DELIMITER = ';'
CSV_QUOTECHAR = '|'
CSV_QUOTING   = csv.QUOTE_MINIMAL
CSV_LINETERMINATOR = '\n'

class writer(object):

    def __init__(self, filestream, uuid4_id=None, columns_headers=None, def_url=None, def_text=None, software=None, update=False):
        self.filestream      = filestream
        self.csvwriter       = csv.writer(filestream, delimiter=CSV_DELIMITER, quotechar=CSV_QUOTECHAR, quoting=CSV_QUOTING, lineterminator=CSV_LINETERMINATOR)
        self.uuid4           = str(uuid.uuid4() if uuid4_id is None else uuid4_id)
        self.columns_headers = columns_headers
        
        if update:
            self.csvwriter.writerow([flags.UPDATED_ON_FLAG, datetime.datetime.now()])
        else:
            self.csvwriter.writerow([flags.UUID4_FLAG, self.uuid4])
            self.csvwriter.writerow([flags.CREATED_ON_FLAG, datetime.datetime.now()])
            
            if def_url:  self.__add_def_url(def_url)
            if def_text: self.__add_def_text(def_text)

        if software: self.csvwriter.writerow([flags.SOFTWARE_REF_FLAG, software])
       
        self._write_header = True if columns_headers else False

    def __add_def_url(self, def_url):
        self.csvwriter.writerow([flags.DEFINITION_URL_FLAG, def_url])

    def __add_def_text(self, def_text):
        self.csvwriter.writerow([flags.DEFINITION_TEXT_FLAG, def_text])

    def add_parent_ref(self, ref_uuid4):
        self.csvwriter.writerow([flags.PARENT_REF_FLAG, ref_uuid4])

    def add_external_ref(self, ref_uuid4):
        self.csvwriter.writerow([flags.EXTERNAL_REF_FLAG, ref_uuid4])

    def add_external_url(self, url):
        self.csvwriter.writerow([flags.EXTERNAL_URL_FLAG, url])

    def add_external_file(self, filepath):
        filename, hash_algorithm, hash_value = hash_utils.calculate_hash(filepath)

        self.csvwriter.writerow([flags.EXTERNAL_FILE_FLAG, filename, hash_algorithm, hash_value])

    def writerow(self, row): 
        if self._write_header: 
            self.csvwriter.writerow([flags.HAS_HEADER_FLAG])
            self.csvwriter.writerow(self.columns_headers)
            self._write_header = False

        self.csvwriter.writerow(row)



    def flush(self):
        self.filestream.flush()






class reader(object):

    def __init__(self, filestream):
        self.csvreader  = csv.reader(filestream, delimiter=CSV_DELIMITER, quotechar=CSV_QUOTECHAR, quoting=CSV_QUOTING, lineterminator=CSV_LINETERMINATOR)
        
        self.uuid4      = None
        self.created_on = None
        self.updated_on = None
        self.parents    = None
        self.externals  = None
        self.software   = None
        self.def_text   = None
        self.def_url    = None
        self.external_urls  = None
        self.external_files = None
        self.columns_headers = None

        self.metadata_countrows = 0 #stores when the row data starts
        self._read_metadata     = True

    def __iter__(self): return self

    def __next__(self):
        row = next(self.csvreader, None)

        if row is None: raise StopIteration
        
        if self._read_metadata:
            self._read_metadata = self.__checkfor_flags(row)
            while( self._read_metadata ):
                # count the number of metadata rows
                self.metadata_countrows +=1
                row = next(self.csvreader, None)
                self._read_metadata = self.__checkfor_flags(row)
                if row is None: raise StopIteration

        return row

    def __checkfor_flags(self, row):
        if len(row)==0: return False
       
        if row[0] == flags.UUID4_FLAG:
            self.uuid4 = row[1]
            return True

        elif row[0] == flags.CREATED_ON_FLAG:
            self.created_on = parser.parse(row[1])
            return True

        elif row[0] == flags.UPDATED_ON_FLAG:         
            if self.updated_on is None: self.updated_on = []
            self.updated_on.append(parser.parse(row[1]))
            return True

        elif row[0] == flags.SOFTWARE_REF_FLAG:
            self.software = row[1]
            return True

        elif row[0] == flags.PARENT_REF_FLAG:
            if self.parents is None: self.parents = []
            self.parents.append(row[1])
            return True

        elif row[0] == flags.EXTERNAL_REF_FLAG:
            if self.externals is None: self.externals = []
            self.externals.append(row[1])
            return True

        elif row[0] == flags.EXTERNAL_URL_FLAG:
            if self.external_urls is None: self.external_urls = []
            self.external_urls.append(row[1:])
            return True

        elif row[0] == flags.EXTERNAL_FILE_FLAG:
            if self.external_files is None: self.external_files = []
            self.external_files.append(row[1:])
            return True

        elif row[0] == flags.DEFINITION_TEXT_FLAG:
            self.def_text = row[1]
            return True

        elif row[0] == flags.DEFINITION_URL_FLAG:
            self.def_url = row[1]
            return True

        elif row[0] == flags.HAS_HEADER_FLAG:
            self.columns_headers = next(self.csvreader, None)
            return True

        return False

    @staticmethod
    def count_metadata_rows(filename):
        count = 0
        with open(filename) as filestream:
            csvreader = csv.reader(filestream, delimiter=CSV_DELIMITER, quotechar=CSV_QUOTECHAR, quoting=CSV_QUOTING, lineterminator=CSV_LINETERMINATOR)
            for i, row in enumerate(csvreader):
                if i>20: break
                
                if row[0] in [
                    flags.UUID4_FLAG,
                    flags.CREATED_ON_FLAG,
                    flags.UPDATED_ON_FLAG,
                    flags.SOFTWARE_REF_FLAG,
                    flags.PARENT_REF_FLAG,
                    flags.EXTERNAL_REF_FLAG,
                    flags.EXTERNAL_FILE_FLAG,
                    flags.EXTERNAL_URL_FLAG,
                    flags.DEFINITION_TEXT_FLAG,
                    flags.DEFINITION_URL_FLAG,
                    flags.HAS_HEADER_FLAG
                ]: 
                    count += 1
                else:
                    return count
        return count





import pyforms
from AnyQt import QtCore, QtGui
from pybpod_alyx_module.module_api import AlyxModule
from pyforms.basewidget import BaseWidget
from pyforms.controls import ControlList


class AlyxDetails(AlyxModule, BaseWidget):

    TITLE = 'Alyx subject details'

    def __init__(self, _subject=None):
        BaseWidget.__init__(self, self.TITLE, parent_win=_subject.mainwindow)
        AlyxModule.__init__(self)

        self._details_list = ControlList('Alyx subject details', readonly=True)
        self._details_list += ('Nickname', _subject.name)
        #self._details_list += ('URL', _subject.alyx_url)
        self._details_list += ('ID', _subject.alyx_id)
        self._details_list += ('Responsible user', _subject.alyx_responsible_user)
        self._details_list += ('Birth date', _subject.alyx_birth_date)
        self._details_list += ('Age (weeks)', _subject.alyx_age_weeks)
        self._details_list += ('Death date', _subject.alyx_death_date)
        self._details_list += ('Species', _subject.alyx_species)
        self._details_list += ('Sex', _subject.alyx_sex)
        self._details_list += ('Litter', _subject.alyx_litter)
        self._details_list += ('Strain', _subject.alyx_strain)
        self._details_list += ('Source', _subject.alyx_source)
        self._details_list += ('Line', _subject.alyx_line)
        self._details_list += ('Projects', ", ".join(map(str, _subject.alyx_projects)) if _subject.alyx_projects else None)
        self._details_list += ('Lab', _subject.alyx_lab)
        self._details_list += ('Genotype', ", ".join(map(str, _subject.alyx_genotype)) if _subject.alyx_genotype else None)
        self._details_list += ('Description', _subject.alyx_description)
        self._details_list += ('Alive', _subject.alyx_alive)
        #self._details_list += ('Weighings', ", ".join(map(str, _subject.alyx_weighings)) if _subject.alyx_weighings else None)
        #self._details_list += ('Reference weight', _subject.alyx_reference_weight)
        #self._details_list += ('Water administrations', ", ".join(map(str, _subject.alyx_water_administrations)) if _subject.alyx_water_administrations else None)
        #self._details_list += ('Last water restriction', _subject.alyx_last_water_restriction)
        #self._details_list += ('Expected water', _subject.alyx_expected_water)
        #self._details_list += ('Remaining water', _subject.alyx_remaining_water)

        self.set_margin(10)
        self.setMinimumWidth(500)
        self.setMinimumHeight(560)

        self.formset = [
            '_details_list'
        ]

    def keyPressEvent(self, event: QtGui.QKeyEvent):
        if event.key() == QtCore.Qt.Key_Escape:
            self.close()


if __name__ == '__main__':
    pyforms.start_app(AlyxDetails, geometry=(0, 0, 300, 300))



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

logger = logging.getLogger(__name__)


class ChannelType(object):
    """
    Define if channel type is input or output.
    These values must be set according to Bpod firmware specification.
    """

    #: Input channel
    INPUT = 1

    #: Output channel
    OUTPUT = 2


class ChannelName(object):
    """
    Available channel names.
    These values must be set according to Bpod firmware specification.
    """

    #: Analog channel with PWM support (e.g. Led)
    PWM = "PWM"

    #: Analog channel for connecting a valve
    VALVE = "Valve"

    #: BNC channel
    BNC = "BNC"

    #: Wire channel
    WIRE = "Wire"

    #: Serial channel
    SERIAL = "Serial"


class EventsPositions(object):
    """

    """

    def __init__(self):
        self.Event_USB = 0  # type: int
        self.Event_Port = 0  # type: int
        self.Event_BNC = 0  # type: int
        self.EventWire = 0  # type: int
        self.globalTimerStart = 0  # type: int
        self.globalTimerEnd = 0  # type: int
        self.globalTimerTrigger = 0  # type: int
        self.globalTimerCancel = 0  # type: int
        self.globalCounter = 0  # type: int
        self.condition = 0  # type: int
        self.jump = 0  # type: int
        self.Tup = 0  # type: int
        self.output_USB = 0  # type: int
        self.output_VALVE = 0  # type: int
        self.output_BNC = 0  # type: int
        self.output_Wire = 0  # type: int
        self.output_PWM = 0  # type: int


class Channels(object):
    """
    Bpod main class
    """

    def __init__(self):
        self.event_names = []
        self.input_channel_names = []
        self.output_channel_names = []
        self.events_positions = EventsPositions()

    def setup_input_channels(self, hardware, modules):
        """
        Generate event and input channel names
        """
        Pos = 0
        nUSB = 0
        nUART = 0
        nBNCs = 0
        nWires = 0
        nPorts = 0

        for i in range(len(hardware.inputs)):
            if hardware.inputs[i] == "U":

                nUART += 1
                module = modules[nUART - 1]
                module_name = ""
                if module.connected:
                    module_name = module.name
                    self.input_channel_names += [module_name]
                else:
                    module_name = "Serial" + str(nUART)
                    self.input_channel_names += [module_name]

                n_module_event_names = len(module.event_names)

                for j in range(module.n_serial_events):
                    if j < n_module_event_names:
                        self.event_names += [module_name + "_" + module.event_names[j]]
                    else:

                        self.event_names += [module_name + "_" + str(j + 1)]
                    Pos += 1

            elif hardware.inputs[i] == "X":
                if nUSB == 0:
                    self.events_positions.Event_USB = Pos
                nUSB += 1
                self.input_channel_names += ["USB" + str(nUSB)]
                loops_n = int(hardware.max_serial_events / (len(modules) + 1))
                for j in range(loops_n):
                    self.event_names += ["SoftCode" + str(j + 1)]
                    Pos += 1
            elif hardware.inputs[i] == "P":
                if nPorts == 0:
                    self.events_positions.Event_Port = Pos
                nPorts += 1
                self.input_channel_names += ["Port" + str(nPorts)]
                self.event_names += [self.input_channel_names[-1] + "In"]
                Pos += 1
                self.event_names += [self.input_channel_names[-1] + "Out"]
                Pos += 1
            elif hardware.inputs[i] == "B":
                if nBNCs == 0:
                    self.events_positions.Event_BNC = Pos
                nBNCs += 1
                self.input_channel_names += ["BNC" + str(nBNCs)]
                self.event_names += [self.input_channel_names[-1] + "High"]
                Pos += 1
                self.event_names += [self.input_channel_names[-1] + "Low"]
                Pos += 1
            elif hardware.inputs[i] == "W":
                if nWires == 0:
                    self.events_positions.Event_Wire = Pos
                nWires += 1
                self.input_channel_names += ["Wire" + str(nWires)]
                self.event_names += [self.input_channel_names[-1] + "High"]
                Pos += 1
                self.event_names += [self.input_channel_names[-1] + "Low"]
                Pos += 1

        self.events_positions.globalTimerStart = Pos
        for i in range(hardware.n_global_timers):
            self.event_names += ["GlobalTimer" + str(i + 1) + "_Start"]
            Pos += 1

        self.events_positions.globalTimerEnd = Pos
        for i in range(hardware.n_global_timers):
            self.event_names += ["GlobalTimer" + str(i + 1) + "_End"]
            self.input_channel_names += ["GlobalTimer" + str(i + 1)]
            Pos += 1

        self.events_positions.globalCounter = Pos
        for i in range(hardware.n_global_counters):
            self.event_names += ["GlobalCounter" + str(i + 1) + "_End"]
            Pos += 1

        self.events_positions.condition = Pos
        for i in range(hardware.n_conditions):
            self.event_names += ["Condition" + str(i + 1)]
            Pos += 1

        self.event_names += ["Tup"]
        self.events_positions.Tup = Pos
        Pos += 1

        logger.debug("event_names: %s", self.event_names)
        logger.debug("events_positions: %s", self.events_positions)

    def setup_output_channels(self, hw_outputs, hardware):
        """
        Generate output channel names
        """
        nUSB = 0
        nUART = 0
        nVALVE = 0
        nBNCs = 0
        nWires = 0
        nPorts = 0
        for i in range(len(hw_outputs)):
            if hw_outputs[i] == "U":
                nUART += 1
                self.output_channel_names += ["Serial" + str(nUART)]

            if hw_outputs[i] == "X":
                if nUSB == 0:
                    self.events_positions.output_USB = len(self.output_channel_names)
                nUSB += 1
                self.output_channel_names += ["SoftCode"]

            if hw_outputs[i] == "V":
                if nVALVE == 0:
                    self.events_positions.output_VALVE = len(self.output_channel_names)
                nVALVE += 1
                self.output_channel_names += [
                    "Valve" + str(nVALVE)
                ]  # Assume an SPI shift register mapping bits of a byte to 8 valves

            if hw_outputs[i] == "B":
                if nBNCs == 0:
                    self.events_positions.output_BNC = len(self.output_channel_names)
                nBNCs += 1
                self.output_channel_names += [
                    "BNC" + str(nBNCs)
                ]  # Assume an SPI shift register mapping bits of a byte to 8 valves

            if hw_outputs[i] == "W":
                if nWires == 0:
                    self.events_positions.output_Wire = len(self.output_channel_names)
                nWires += 1
                self.output_channel_names += [
                    "Wire" + str(nWires)
                ]  # Assume an SPI shift register mapping bits of a byte to 8 valves

            if hw_outputs[i] == "P":
                if nPorts == 0:
                    self.events_positions.output_PWM = len(self.output_channel_names)
                nPorts += 1
                self.output_channel_names += [
                    "PWM" + str(nPorts)
                ]  # Assume an SPI shift register mapping bits of a byte to 8 valves

        self.output_channel_names += ["GlobalTimerTrig"]
        self.events_positions.globalTimerTrigger = len(self.output_channel_names) - 1
        self.output_channel_names += ["GlobalTimerCancel"]
        self.events_positions.globalTimerCancel = len(self.output_channel_names) - 1
        self.output_channel_names += ["GlobalCounterReset"]

        logger.debug("output_channel_names: %s", self.output_channel_names)

    def get_event_name(self, event_idx):
        """

        :param event_idx:
        :return:
        """

        try:
            event_name = self.event_names[event_idx]
        except IndexError:
            event_name = "unknown event name"

        return event_name

    def __str__(self):

        buff = "\n****************** EVENTS ******************\n"
        for idx, event in enumerate(self.event_names):
            buff += "{0: >3} : {1: <24}".format(idx, event)
            if ((idx + 1) % 3) == 0 and idx != 0:
                buff += "\n"

        buff += "\n\n****************** INPUT CHANNELS ******************\n"
        for idx, channel in enumerate(self.input_channel_names):
            buff += "{0: >3} : {1: <24}".format(idx, channel)
            if ((idx + 1) % 3) == 0 and idx != 0:
                buff += "\n"

        buff += "\n\n****************** OUTPUT CHANNELS ******************\n"
        for idx, channel in enumerate(self.output_channel_names):
            buff += "{0: >3} : {1: <24}".format(idx, channel)
            if ((idx + 1) % 3) == 0 and idx != 0:
                buff += "\n"

        return "SMA Channels\n" + buff + "\n\n"



# !/usr/bin/python3
# -*- coding: utf-8 -*-

from confapp import conf

from pyforms_generic_editor.models.project.generic_project import GenericProject

Project = type(
    'Project',
    tuple(conf.GENERIC_EDITOR_PLUGINS_FINDER.find_class('models.project.Project') + [GenericProject]),
    {}
)



from pyforms.basewidget import BaseWidget
from pyforms.controls import ControlText
from pybpodgui_api.models.user import User


class UserWindow(User, BaseWidget):

    def __init__(self, project=None):

        BaseWidget.__init__(self, 'User')
        self._namebox = ControlText('Name')
        User.__init__(self, project)

        # print(self._name)

        self.layout().setContentsMargins(5, 10, 5, 5)

        self._formset = [
            '_namebox'
        ]

        self._namebox.value = self._name

        self._namebox.changed_event = self.__name_changed_evt

    def __name_changed_evt(self):
        if not hasattr(self, '_update_name') or not self._update_name:
            self.name = self._namebox.value

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._update_name = True  # Flag to avoid recursive calls when editing the name text field
        self._name = value
        self._namebox.value = value
        self._update_name = False



from confapp import conf
from AnyQt.QtGui import QIcon

from pybpodgui_plugin_waveplayer.module_gui import WavePlayerModuleGUI

class ProjectsWavePlayer(object):

	def register_on_main_menu(self, mainmenu):
		super().register_on_main_menu(mainmenu)

		if len([m for m in mainmenu if 'Tools' in m.keys()]) == 0:
			mainmenu.append({'Tools': []})

		menu_index = 0
		for i, m in enumerate(mainmenu):
			if 'Tools' in m.keys(): menu_index=i; break

		mainmenu[menu_index]['Tools'].append( '-' )	
		mainmenu[menu_index]['Tools'].append( {'Wave player': self.open_waveplayer_plugin, 'icon': None})
	
	def open_waveplayer_plugin(self):
		if not hasattr(self, 'rotaryencoder_plugin'):
			self.waveplayer_plugin = WavePlayerModuleGUI(self)
			self.waveplayer_plugin.show()
			#self.rotaryencoder_plugin.resize(*conf.ROTARYENCODER_PLUGIN_WINDOW_SIZE)			
		else:
			self.waveplayer_plugin.show()

		return self.waveplayer_plugin


from pybpodapi.bpod_modules.bpod_module import BpodModule


class RotaryEncoder(BpodModule):
    """
    .. note:: This API was based on the `Rotary Encoder board documentation <https://sites.google.com/site/bpoddocumentation/bpod-user-guide/serial-interfaces/rotaryencodermodule>`_.
    """

    COM_TOGGLEOUTPUTSTREAM      = ord('O')
    COM_STOP_STREAMANDLOGGING   = ord('X')
    COM_ENABLE_ALLTHRESHOLDS    = ord('E')
    COM_SETZEROPOS              = ord('Z')
    COM_START_LOGGING           = ord('L')
    COM_STOP_LOGGING            = ord('F')
    COM_TIMESTAMP_BYTE          = ord('#')


    @staticmethod
    def check_module_type(module_name):
        return module_name and module_name.startswith('RotaryEncoder')


    def create_resetpositions_trigger(self):
        """
        Create a trigger to reset the positions and the threshods
        :return: trigger_id
        """
        return self.load_message([ self.COM_SETZEROPOS, self.COM_ENABLE_ALLTHRESHOLDS])


    def activate_outputstream(self):
        """
        Activate module output stream.
        """
        self.write_char_array([self.COM_TOGGLEOUTPUTSTREAM,1])

    def deactivate_outputstream(self):
        """
        Deactivate module output stream.
        """
        self.write_char_array([self.COM_TOGGLEOUTPUTSTREAM,0])

    def stop_streaming_and_logging(self):
        """
        Stops streaming + logging.
        """
        self.write_char_array([self.COM_STOP_STREAMANDLOGGING])

    def enable_positions_threshold(self):
        """
        Enable all position thresholds.
        """
        self.write_char_array([self.COM_ENABLE_ALLTHRESHOLDS])

    def set_position_zero(self):
        """
        Set current rotary encoder position to zero. 
        """
        self.write_char_array([self.COM_SETZEROPOS])

    def starts_logging(self):
        """
        Start logging position+time data to the microSD card. 
        """
        self.write_char_array([self.COM_START_LOGGING])

    def stops_logging(self):
        """
        Finish logging position+time data to the microSD card. 
        """
        self.write_char_array([self.COM_STOP_LOGGING])

    def timestamp_byte(self, value):
        """
        value as to be a byte
        """
        self.write_char_array([self.COM_TIMESTAMP_BYTE, value])


# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
import os
import pybpodgui_api
from sca.formats import json
from pybpodgui_api.models.task.task_base import TaskBase

logger = logging.getLogger(__name__)


class TaskIO(TaskBase):
    """
    Task I/O operations
    """

    def __init__(self, project=None):
        super(TaskIO, self).__init__(project)

        self.data = None

    ##########################################################################
    ####### FUNCTIONS ########################################################
    ##########################################################################

    def save(self):
        """
        Save setup data on filesystem.

        :ivar str project_path: Project path.
        :ivar dict data: Dictionary where to save the data to.
        :return: Dictionary containing the task info to save.
        :rtype: dict
        """

        if (self.path and not os.path.exists(self.path)) or not self.path:
            self.make_path()

        if (self.filepath and not os.path.exists(self.filepath)) or not self.filepath:
            self.filepath = self.make_emptyfile()

        """
        current_path     = os.path.dirname(self.filepath)
        current_filename = os.path.basename(self.filepath)
        future_path      = self.path

        if current_path!=future_path:
            shutil.move( current_path, future_path )
            current_filepath = os.path.join(future_path, current_filename)
            future_filepath  = os.path.join(future_path, self.name+'.py')
            shutil.move( current_filepath, future_filepath )
        """

        if self.data:
            data = self.data
        else:
            data = json.scadict(
                uuid4_id=self.uuid4,
                software='PyBpod GUI API v'+str(pybpodgui_api.__version__),
                def_url='http://pybpod.readthedocs.org',
                def_text='This file contains information about a PyBpod protocol.'
            )
        data['name'] = self.name
        data['trigger-softcodes'] = self.trigger_softcodes
        data['commands'] = [cmd.save() for cmd in self.commands]

        config_path = os.path.join(self.path, self.name+'.json')
        with open(config_path, 'w') as fstream:
            json.dump(data, fstream)

    def load(self, path):
        """
        Load setup data from filesystem

        :ivar str task_path: Path of the task
        :ivar dict data: data object that contains all task info
        """
        self.name = os.path.basename(path)

        config_path = os.path.join(self.path, self.name+'.json')
        if os.path.exists(config_path):
            with open(config_path, 'r') as stream:
                self.data = data = json.load(stream)
                self.uuid4 = data.uuid4 if data.uuid4 else self.uuid4
                self.filepath = os.path.join(self.path, self.name+'.py')
        else:
            self.data = data = {}

        self.trigger_softcodes = data.get('trigger-softcodes', None)

        for cmddata in data.get('commands', []):
            cmd = getattr(self, cmddata['type'])()
            cmd.load(cmddata)

    def collect_data(self, data):
        data.update({'name': self.name})
        data.update({'trigger_softcodes': self.trigger_softcodes})

        data.update({'commands': []})

        for cmd in self.commands:
            data['commands'].append(cmd.collect_data({}))

        return data



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

from confapp import conf as settings
from pybpodapi.bpod.bpod_base import BpodBase
from pybpodapi.bpod.hardware.channels import ChannelType
from pybpodapi.bpod_modules.bpod_module import BpodModule
from pybpodapi.com.arcom import ArCOM, ArduinoTypes
from pybpodapi.com.protocol.recv_msg_headers import ReceiveMessageHeader
from pybpodapi.com.protocol.send_msg_headers import SendMessageHeader
from pybpodapi.exceptions.bpod_error import BpodErrorException

logger = logging.getLogger(__name__)


class BpodCOMProtocol(BpodBase):
    """
    Define command actions that can be requested to Bpod device.

    **Private attributes**

        _arcom
            :class:`pybpodapi.com.arcom.ArCOM`

            ArCOM object that performs serial communication.

    **Methods**

    """

    def __init__(self, serial_port=None, sync_channel=None, sync_mode=None):
        super(BpodCOMProtocol, self).__init__(serial_port, sync_channel, sync_mode)

        self._arcom = None  # type: ArCOM
        self.bpod_com_ready = False

        # used to keep the list of msg ids sent using the load_serial_message function
        self.msg_id_list = [False for i in range(255)]

        if self.serial_port:
            self.open()

    def open(self):
        super(BpodCOMProtocol, self).open()
        self.bpod_com_ready = True

    def close(self):
        if self.bpod_com_ready:
            super(BpodCOMProtocol, self).close()
            self._arcom.close()
            self.bpod_com_ready = False

    def manual_override(self, channel_type, channel_name, channel_number, value):
        """
        Manually override a Bpod channel

        :param ChannelType channel_type: channel type input or output
        :param ChannelName channel_name: channel name like PWM, Valve, etc.
        :param channel_number:
        :param int value: value to write on channel
        """
        if channel_type == ChannelType.INPUT:
            input_channel_name = channel_name + str(channel_number)
            channel_number = self.hardware.channels.input_channel_names.index(input_channel_name)
            try:
                self._bpodcom_override_input_state(channel_number, value)
            except:
                raise BpodErrorException(
                    'Error using manual_override: {name} is not a valid channel name.'.format(name=channel_name))

        elif channel_type == ChannelType.OUTPUT:
            if channel_name == 'Serial':
                self._bpodcom_send_byte_to_hardware_serial(channel_number, value)

            else:
                try:
                    output_channel_name = channel_name + str(channel_number)
                    channel_number = self.hardware.channels.output_channel_names.index(output_channel_name)
                    self._bpodcom_override_digital_hardware_state(channel_number, value)
                except:
                    raise BpodErrorException('Error using manual_override: {name} is not a valid channel name.'.format(
                        name=output_channel_name))
        else:
            raise BpodErrorException('Error using manualOverride: first argument must be "Input" or "Output".')

    def _bpodcom_connect(self, serial_port, baudrate=115200, timeout=1):
        """
        Connect to Bpod using serial connection

        :param str serial_port: serial port to connect
        :param int baudrate: baudrate for serial connection
        :param float timeout: timeout which controls the behavior of read()
        """
        logger.debug("Connecting on port: %s", serial_port)
        self._arcom = ArCOM().open(serial_port, baudrate, timeout)

    def _bpodcom_disconnect(self):
        """
        Signal Bpod device to disconnect now
        """
        logger.debug("Requesting disconnect (%s)", SendMessageHeader.DISCONNECT)

        self._arcom.write_char(SendMessageHeader.DISCONNECT)

        res = self._arcom.read_byte() == b'1'

        logger.debug("Disconnect result (%s)", str(res))
        return res

    # def __bpodcom_check_com_ready(self):
    #    if not self.bpod_com_ready: self.open()

    def _bpodcom_handshake(self):
        """
        Test connectivity by doing an handshake

        :return: True if handshake received, False otherwise
        :rtype: bool
        """

        logger.debug("Requesting handshake (%s)", SendMessageHeader.HANDSHAKE)

        self._arcom.write_char(SendMessageHeader.HANDSHAKE)

        response = self._arcom.read_char()  # Receive response

        logger.debug("Response command is: %s", response)

        return True if response == ReceiveMessageHeader.HANDSHAKE_OK else False

    def _bpodcom_firmware_version(self):
        """
        Request firmware and machine type from Bpod

        :return: firmware and machine type versions
        :rtype: int, int
        """

        logger.debug("Requesting firmware version (%s)", SendMessageHeader.FIRMWARE_VERSION)

        self._arcom.write_char(SendMessageHeader.FIRMWARE_VERSION)

        fw_version = self._arcom.read_uint16()  # type: int
        machine_type = self._arcom.read_uint16()  # type: int

        logger.debug("Firmware version: %s", fw_version)
        logger.debug("Machine type: %s", machine_type)

        return fw_version, machine_type

    def _bpodcom_reset_clock(self):
        """
        Reset session clock
        """
        logger.debug("Resetting clock")

        self._arcom.write_char(SendMessageHeader.RESET_CLOCK)
        return self._arcom.read_byte() == bytes(1)

    def _bpodcom_stop_trial(self):
        """
        Stops ongoing trial (We recommend using computer-side pauses between trials, to keep data uniform)
        """
        logger.debug("Pausing trial")
        self._arcom.write_char(SendMessageHeader.EXIT_AND_RETURN)

    def _bpodcom_pause_trial(self):
        """
        Pause ongoing trial (We recommend using computer-side pauses between trials, to keep data uniform)
        """
        logger.debug("Pausing trial")
        bytes2send = ArduinoTypes.get_uint8_array([ord(SendMessageHeader.PAUSE_TRIAL), 0])
        self._arcom.write_array(bytes2send)

    def _bpodcom_resume_trial(self):
        """
        Resumes ongoing trial (We recommend using computer-side pauses between trials, to keep data uniform)
        """
        logger.debug("Resume trial")
        bytes2send = ArduinoTypes.get_uint8_array([ord(SendMessageHeader.PAUSE_TRIAL), 1])
        self._arcom.write_array(bytes2send)

    def _bpodcom_get_timestamp_transmission(self):
        """
        Return timestamp transmission scheme
        """
        logger.debug("Get timestamp transmission")

        self._arcom.write_char(SendMessageHeader.GET_TIMESTAMP_TRANSMISSION)
        return self._arcom.read_byte()

    def _bpodcom_hardware_description(self, hardware):
        """
        Request hardware description from Bpod

        :param Hardware hardware: hardware
        """

        logger.debug("Requesting hardware description (%s)...", SendMessageHeader.HARDWARE_DESCRIPTION)
        self._arcom.write_char(SendMessageHeader.HARDWARE_DESCRIPTION)

        max_states = self._arcom.read_uint16()  # type: int
        logger.debug("Read max states: %s", max_states)

        cycle_period = self._arcom.read_uint16()  # type: int
        logger.debug("Read cycle period: %s", cycle_period)

        max_serial_events = self._arcom.read_uint8()  # type: int
        logger.debug("Read number of events per serial channel: %s", max_serial_events)

        n_global_timers = self._arcom.read_uint8()  # type: int
        logger.debug("Read number of global timers: %s", n_global_timers)

        n_global_counters = self._arcom.read_uint8()  # type: int
        logger.debug("Read number of global counters: %s", n_global_counters)

        n_conditions = self._arcom.read_uint8()  # type: int
        logger.debug("Read number of conditions: %s", n_conditions)

        n_inputs = self._arcom.read_uint8()  # type: int
        logger.debug("Read number of inputs: %s", n_inputs)

        inputs = self._arcom.read_char_array(array_len=n_inputs)  # type: list(str)
        logger.debug("Read inputs: %s", inputs)

        n_outputs = self._arcom.read_uint8()  # type: int
        logger.debug("Read number of outputs: %s", n_outputs)

        outputs = self._arcom.read_char_array(array_len=n_outputs)  # type: list(str)
        logger.debug("Read outputs: %s", outputs)

        hardware.max_states = max_states
        hardware.cycle_period = cycle_period
        hardware.max_serial_events = max_serial_events
        hardware.n_global_timers = n_global_timers
        hardware.n_global_counters = n_global_counters
        hardware.n_conditions = n_conditions
        hardware.inputs = inputs
        hardware.outputs = outputs  # + ['G', 'G', 'G']

        hardware.live_timestamps = self._bpodcom_get_timestamp_transmission()

    def _bpodcom_enable_ports(self, hardware):
        """
        Enable input ports on Bpod device

        :param list[int] inputs_enabled: list of inputs to be enabled (0 = disabled, 1 = enabled)
        :rtype: bool
        """

        ###### set inputs enabled or disabled #######################################################
        hardware.inputs_enabled = [0] * len(hardware.inputs)

        for j, i in enumerate(hardware.bnc_inputports_indexes):
            hardware.inputs_enabled[i] = settings.BPOD_BNC_PORTS_ENABLED[j]

        for j, i in enumerate(hardware.wired_inputports_indexes):
            hardware.inputs_enabled[i] = settings.BPOD_WIRED_PORTS_ENABLED[j]

        for j, i in enumerate(hardware.behavior_inputports_indexes):
            hardware.inputs_enabled[i] = settings.BPOD_BEHAVIOR_PORTS_ENABLED[j]
        #############################################################################################

        logger.debug("Requesting ports enabling (%s)", SendMessageHeader.ENABLE_PORTS)
        logger.debug("Inputs enabled (%s): %s", len(hardware.inputs_enabled), hardware.inputs_enabled)

        bytes2send = ArduinoTypes.get_uint8_array([ord(SendMessageHeader.ENABLE_PORTS)] + hardware.inputs_enabled)

        self._arcom.write_array(bytes2send)

        response = self._arcom.read_uint8()  # type: int

        logger.debug("Response: %s", response)

        return True if response == ReceiveMessageHeader.ENABLE_PORTS_OK else False

    def _bpodcom_set_sync_channel_and_mode(self, sync_channel, sync_mode):
        """
        Request sync channel and sync mode configuration

        :param int sync_channel: 255 = no sync, otherwise set to a hardware channel number
        :param int sync_mode: 0 = flip logic every trial, 1 = every state
        :rtype: bool
        """

        logger.debug("Requesting sync channel and mode (%s)", SendMessageHeader.SYNC_CHANNEL_MODE)

        bytes2send = ArduinoTypes.get_uint8_array([ord(SendMessageHeader.SYNC_CHANNEL_MODE), sync_channel, sync_mode])

        self._arcom.write_array(bytes2send)

        response = self._arcom.read_uint8()  # type: int

        logger.debug("Response: %s", response)

        return True if response == ReceiveMessageHeader.SYNC_CHANNEL_MODE_OK else False

    def _bpodcom_echo_softcode(self, softcode):
        """
        Send soft code
        """
        logger.debug("Echo softcode")
        self._arcom.write_char(SendMessageHeader.ECHO_SOFTCODE)
        self._arcom.write_char(softcode)

        bytes2send = ArduinoTypes.get_uint8_array([ord(SendMessageHeader.ECHO_SOFTCODE), softcode])
        self._arcom.write_array(bytes2send)

    def _bpodcom_manual_override_exec_event(self, event_index, event_data):
        """
        Send soft code
        """
        logger.debug("Manual override execute virtual event")
        bytes2send = ArduinoTypes.get_uint8_array([ord(SendMessageHeader.MANUAL_OVERRIDE_EXEC_EVENT), event_index, event_data])
        self._arcom.write_array(bytes2send)

    def _bpodcom_override_input_state(self, channel_number, value):
        """
        Manually set digital value on channel

        :param int channel_number: number of Bpod port
        :param int value: value to be written
        """
        logger.debug("Override input state")

        bytes2send = ArduinoTypes.get_uint8_array(
            [ord(SendMessageHeader.MANUAL_OVERRIDE_EXEC_EVENT), channel_number, value])
        self._arcom.write_array(bytes2send)

    def _bpodcom_send_softcode(self, softcode):
        """
        Send soft code
        """
        logger.debug("Send softcode")
        bytes2send = ArduinoTypes.get_uint8_array([ord(SendMessageHeader.TRIGGER_SOFTCODE), softcode])
        self._arcom.write_array(bytes2send)

    def _bpodcom_send_state_machine(self, message):
        """
        Sends state machine to Bpod

        :param list(int) message: TODO
        :param list(int) ThirtyTwoBitMessage: TODO
        """
        # self.__bpodcom_check_com_ready()

        self._arcom.write_array(message)

    def _bpodcom_run_state_machine(self):
        """
        Request to run state machine now
        """
        # self.__bpodcom_check_com_ready()

        logger.debug("Requesting state machine run (%s)", SendMessageHeader.RUN_STATE_MACHINE)

        self._arcom.write_char(SendMessageHeader.RUN_STATE_MACHINE)

    def _bpodcom_get_trial_timestamp_start(self):
        data = self._arcom.read_bytes_array(8)
        self.trial_start_micros = ArduinoTypes.cvt_uint64(b''.join(data))
        return self.trial_start_micros / float(self.hardware.DEFAULT_FREQUENCY_DIVIDER)

    def _bpodcom_read_trial_start_timestamp_seconds(self):
        """
        A new incoming timestamp message is available. Read trial start timestamp in millseconds and convert to seconds.

        :return: trial start timestamp in milliseconds
        :rtype: float
        """
        response = self._arcom.read_uint32()  # type: int

        # print('response', response)
        # logger.debug("Received start trial timestamp in millseconds: %s", response)

        # trial_start_timestamp = response / 1000.0

        return response * self.hardware.times_scale_factor

    def _bpodcom_read_timestamps(self):

        data = self._arcom.read_bytes_array(12)

        n_hw_timer_cyles = ArduinoTypes.cvt_uint32(b''.join(data[:4]))
        trial_end_micros = ArduinoTypes.cvt_uint64(b''.join(data[4:12]))  # / float(self.hardware.DEFAULT_FREQUENCY_DIVIDER)
        trial_end_timestamp = trial_end_micros / float(self.hardware.DEFAULT_FREQUENCY_DIVIDER)
        trial_time_from_micros = trial_end_timestamp - self.trial_start_timestamp
        trial_time_from_cycles = n_hw_timer_cyles/self.hardware.cycle_frequency
        discrepancy = abs(trial_time_from_micros - trial_time_from_cycles)*1000

        return trial_end_timestamp, discrepancy

    def _bpodcom_state_machine_installation_status(self):
        """
        Confirm if new state machine was correctly installed

        :rtype: bool
        """
        # self.__bpodcom_check_com_ready()

        response = self._arcom.read_uint8()  # type: int

        logger.debug("Read state machine installation status: %s", response)

        return True if response == ReceiveMessageHeader.STATE_MACHINE_INSTALLATION_STATUS else False

    def data_available(self):
        """
        Finds out if there is data received from Bpod

        :rtype: bool
        """
        return self._arcom.bytes_available() > 0

    def _bpodcom_read_opcode_message(self):
        """
        A new incoming opcode message is available. Read opcode code and data.

        :return: opcode and data
        :rtype: tuple(int, int)
        """
        response = self._arcom.read_uint8_array(array_len=2)
        opcode = response[0]
        data = response[1]

        logger.debug("Received opcode message: opcode=%s, data=%s", opcode, data)

        return opcode, data

    def _bpodcom_read_alltimestamps(self):
        """
        A new incoming timestamps message is available.
        Read number of timestamps to be sent and then read timestamps array.

        :return: timestamps array
        :rtype: list(float)
        """
        n_timestamps = self._arcom.read_uint16()  # type: int

        timestamps = self._arcom.read_uint32_array(array_len=n_timestamps)

        logger.debug("Received timestamps: %s", timestamps)

        return timestamps

    def _bpodcom_read_current_events(self, n_events):
        """
        A new incoming events message is available.
        Read number of timestamps to be sent and then read timestamps array.

        :param int n_events: number of events to read
        :return: a list with events
        :rtype: list(int)
        """
        current_events = self._arcom.read_uint8_array(array_len=n_events)

        logger.debug("Received current events: %s", current_events)

        return current_events

    def _bpodcom_read_event_timestamp(self):
        v = self._arcom.read_uint32()
        return float(v) * self.hardware.times_scale_factor

    def _bpodcom_load_serial_message(self, serial_channel, message_id, serial_message, n_messages):
        """
        Load serial message on channel

        :param TODO
        :rtype: bool
        """
        # self.__bpodcom_check_com_ready()

        if isinstance(serial_channel, BpodModule):
            serial_channel = serial_channel.serial_port

        self.msg_id_list[message_id] = True

        if len(serial_message) > 3:
            raise BpodErrorException('Error: Serial messages cannot be more than 3 bytes in length.')

        if not (1 <= message_id <= 255):
            raise BpodErrorException('Error: Bpod can only store 255 serial messages (indexed 1-255). You used the message_id {0}'.format(message_id))

        message_container = [serial_channel-1, n_messages, message_id, len(serial_message)] + serial_message

        logger.debug("Requesting load serial message (%s)", SendMessageHeader.LOAD_SERIAL_MESSAGE)
        logger.debug("Message: %s", message_container)

        bytes2send = ArduinoTypes.get_uint8_array([ord(SendMessageHeader.LOAD_SERIAL_MESSAGE)] + message_container)

        self._arcom.write_array(bytes2send)

        response = self._arcom.read_uint8()  # type: int

        logger.debug("Confirmation: %s", response)

        return True if response == ReceiveMessageHeader.LOAD_SERIAL_MESSAGE_OK else False

    def _bpodcom_reset_serial_messages(self):
        """
        Reset serial messages on Bpod device

        :rtype: bool
        """
        logger.debug("Requesting serial messages reset (%s)", SendMessageHeader.RESET_SERIAL_MESSAGES)

        self._arcom.write_char(SendMessageHeader.RESET_SERIAL_MESSAGES)

        response = self._arcom.read_uint8()  # type: int

        logger.debug("Confirmation: %s", response)

        return True if response == ReceiveMessageHeader.RESET_SERIAL_MESSAGES else False

    def _bpodcom_override_digital_hardware_state(self, channel_number, value):
        """
        Manually set digital value on channel

        :param int channel_number: number of Bpod port
        :param int value: value to be written
        """

        bytes2send = ArduinoTypes.get_uint8_array(
            [ord(SendMessageHeader.OVERRIDE_DIGITAL_HW_STATE), channel_number, value])
        self._arcom.write_array(bytes2send)

    def _bpodcom_send_byte_to_hardware_serial(self, channel_number, value):
        """
        Send byte to hardware serial channel 1-3

        :param int channel_number:
        :param int value: value to be written
        """
        bytes2send = ArduinoTypes.get_uint8_array(
            [ord(SendMessageHeader.SEND_TO_HW_SERIAL), channel_number, value]
        )
        self._arcom.write_array(bytes2send)

    @property
    def hardware(self):
        # self.__bpodcom_check_com_ready()
        return BpodBase.hardware.fget(self)  # type: Hardware

    @property
    def modules(self):
        # self.__bpodcom_check_com_ready()
        return BpodBase.modules.fget(self)

    # @property
    # def inputs(self):
    #   self.__bpodcom_check_com_ready()
    #   return BpodBase.inputs.fget(self)

    # @property
    # def outputs(self):
    #   self.__bpodcom_check_com_ready()
    #   return BpodBase.outputs.fget(self)

    # @property
    # def channels(self):
    #   self.__bpodcom_check_com_ready()
    #   return BpodBase.channels.fget(self)

    # @property
    # def max_states(self):
    #   self.__bpodcom_check_com_ready()
    #   return BpodBase.max_states.fget(self)

    # @property
    # def max_serial_events(self):
    #   self.__bpodcom_check_com_ready()
    #   return BpodBase.max_serial_events.fget(self)

    # @property
    # def inputs_enabled(self):
    #   self.__bpodcom_check_com_ready()
    #   return BpodBase.inputs_enabled.fget(self)

    # @property
    # def cycle_period(self):
    #   self.__bpodcom_check_com_ready()
    #   return BpodBase.cycle_period.fget(self)

    # @property
    # def n_global_timers(self):
    #   self.__bpodcom_check_com_ready()
    #   return BpodBase.n_global_timers.fget(self)

    # @property
    # def n_global_counters(self):
    #   self.__bpodcom_check_com_ready()
    #   return BpodBase.n_global_counters.fget(self)

    # @property
    # def n_conditions(self):
    #   self.__bpodcom_check_com_ready()
    #   return BpodBase.n_conditions.fget(self)

    # @property
    # def n_uart_channels(self):
    #   self.__bpodcom_check_com_ready()
    #   return BpodBase.n_uart_channels.fget(self)

    # @property
    # def firmware_version(self):
    #   return BpodBase.firmware_version.fget(self)

    # @property
    # def machine_type(self):
    #   self.__bpodcom_check_com_ready()
    #   return BpodBase.machine_type.fget(self)

    # @property
    # def cycle_frequency(self):
    #   self.__bpodcom_check_com_ready()
    #   return BpodBase.cycle_frequency.fget(self)



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
from pybpodapi.bpod.hardware.events.base_eventname import BaseEventName

logger = logging.getLogger(__name__)


class EventName(BaseEventName):
    """
    Input event codes
    These values must be set according to Bpod firmware specification.
    """

    #: Serial1_1
    Serial1_1 = "Serial1_1"

    #: Serial1_2
    Serial1_2 = "Serial1_2"

    #: Serial1_3
    Serial1_3 = "Serial1_3"

    #: Serial1_4
    Serial1_4 = "Serial1_4"

    #: Serial1_5
    Serial1_5 = "Serial1_5"

    #: Serial1_6
    Serial1_6 = "Serial1_6"

    #: Serial1_7
    Serial1_7 = "Serial1_7"

    #: Serial1_8
    Serial1_8 = "Serial1_8"

    #: Serial1_9
    Serial1_9 = "Serial1_9"

    #: Serial1_10
    Serial1_10 = "Serial1_10"

    #: Serial2_1
    Serial2_1 = "Serial2_1"

    #: Serial2_2
    Serial2_2 = "Serial2_2"

    #: Serial2_3
    Serial2_3 = "Serial2_3"

    #: Serial2_4
    Serial2_4 = "Serial2_4"

    #: Serial2_5
    Serial2_5 = "Serial2_5"

    #: Serial2_6
    Serial2_6 = "Serial2_6"

    #: Serial2_7
    Serial2_7 = "Serial2_7"

    #: Serial2_8
    Serial2_8 = "Serial2_8"

    #: Serial2_9
    Serial2_9 = "Serial2_9"

    #: Serial2_10
    Serial2_10 = "Serial2_10"

    #: Serial3_1
    Serial3_1 = "Serial3_1"

    #: Serial3_2
    Serial3_2 = "Serial3_2"

    #: Serial3_3
    Serial3_3 = "Serial3_3"

    #: Serial3_4
    Serial3_4 = "Serial3_4"

    #: Serial3_5
    Serial3_5 = "Serial3_5"

    #: Serial3_6
    Serial3_6 = "Serial3_6"

    #: Serial3_7
    Serial3_7 = "Serial3_7"

    #: Serial3_8
    Serial3_8 = "Serial3_8"

    #: Serial3_9
    Serial3_9 = "Serial3_9"

    #: Serial3_10
    Serial3_10 = "Serial3_10"

    #: SoftCode1
    SoftCode1 = "SoftCode1"

    #: SoftCode2
    SoftCode2 = "SoftCode2"

    #: SoftCode3
    SoftCode3 = "SoftCode3"

    #: SoftCode4
    SoftCode4 = "SoftCode4"

    #: SoftCode5
    SoftCode5 = "SoftCode5"

    #: SoftCode6
    SoftCode6 = "SoftCode6"

    #: SoftCode7
    SoftCode7 = "SoftCode7"

    #: SoftCode8
    SoftCode8 = "SoftCode8"

    #: SoftCode9
    SoftCode9 = "SoftCode9"

    #: SoftCode10
    SoftCode10 = "SoftCode10"

    #: BNC1In
    BNC1In = "BNC1In"

    #: BNC1Out
    BNC1Out = "BNC1Out"

    #: BNC2In
    BNC2In = "BNC2In"

    #: BNC2Out
    BNC2Out = "BNC2Out"

    #: Wire1In
    Wire1In = "Wire1In"

    #: Wire1Out
    Wire1Out = "Wire1Out"

    #: Wire2In
    Wire2In = "Wire2In"

    #: Wire2Out
    Wire2Out = "Wire2Out"

    #: Input port 1
    Port1In = "Port1In"

    #: Output port 1
    Port1Out = "Port1Out"

    #: Input port 2
    Port2In = "Port2In"

    #: Output port 2
    Port2Out = "Port2Out"

    #: Input port 3
    Port3In = "Port3In"

    #: Output port 3
    Port3Out = "Port3Out"

    #: Input port 4
    Port4In = "Port4In"

    #: Output port 4
    Port4Out = "Port4Out"

    #: Input port 5
    Port5In = "Port5In"

    #: Output port 5
    Port5Out = "Port5Out"

    #: Input port 6
    Port6In = "Port6In"

    #: Output port 6
    Port6Out = "Port6Out"

    #: Input port 7
    Port7In = "Port7In"

    #: Output port 7
    Port7Out = "Port7Out"

    #: Input port 8
    Port8In = "Port8In"

    #: Output port 8
    Port8Out = "Port8Out"

    #: GlobalTimer1_Start
    GlobalTimer1_Start = "GlobalTimer1_Start"

    #: GlobalTimer2_Start
    GlobalTimer2_Start = "GlobalTimer2_Start"

    #: GlobalTimer3_Start
    GlobalTimer3_Start = "GlobalTimer3_Start"

    #: GlobalTimer4_Start
    GlobalTimer4_Start = "GlobalTimer4_Start"

    #: GlobalTimer5_Start
    GlobalTimer5_Start = "GlobalTimer5_Start"

    #: GlobalTimer1_End
    GlobalTimer1_End = "GlobalTimer1_End"

    #: GlobalTimer2_End
    GlobalTimer2_End = "GlobalTimer2_End"

    #: GlobalTimer3_End
    GlobalTimer3_End = "GlobalTimer3_End"

    #: GlobalTimer4_End
    GlobalTimer4_End = "GlobalTimer4_End"

    #: GlobalTimer5_End
    GlobalTimer5_End = "GlobalTimer5_End"

    #: GlobalCounter1_End
    GlobalCounter1_End = "GlobalCounter1_End"

    #: GlobalCounter2_End
    GlobalCounter2_End = "GlobalCounter2_End"

    #: GlobalCounter3_End
    GlobalCounter3_End = "GlobalCounter3_End"

    #: GlobalCounter4_End
    GlobalCounter4_End = "GlobalCounter4_End"

    #: GlobalCounter5_End
    GlobalCounter5_End = "GlobalCounter5_End"

    #: Condition1
    Condition1 = "Condition1"

    #: Condition2
    Condition2 = "Condition2"

    #: Condition3
    Condition3 = "Condition3"

    #: Condition4
    Condition4 = "Condition4"

    #: Condition5
    Condition5 = "Condition5"

    #: Serial1Jump
    Serial1Jump = "Serial1Jump"

    #: Serial2Jump
    Serial2Jump = "Serial2Jump"

    #: Serial3Jump
    Serial3Jump = "Serial3Jump"

    #: SoftJump
    SoftJump = "SoftJump"

    #: Tup
    Tup = "Tup"



import requests
import json
from confapp import conf

ENDPOINT = conf.ALYX_PLUGIN_ADDRESS+'/subjects'

class Post():

    def __init__(self,_apibase):
        self.apibase = _apibase     

    
    def newsubject(self, _nickname, _responsible_user, _birth_date = None, _death_date = None, _species = None, _sex = None, _litter = None, _strain = None, _line = None, _description = None, _genotype = None):
        _data = dict(nickname = _nickname,
                    responsible_user = _responsible_user,
                    birth_date = _birth_date,
                    death_date = _death_date,
                    species = _species,
                    sex = _sex,
                    litter = _litter,
                    strain = _strain,
                    line = _line,
                    description = _description,
                    genotype = _genotype)

        print(_data)

        #requests.post(ENDPOINT,headers = self.apibase.headers, data= _data)

    def updatesubject(self, _nickname, _responsible_user, _birth_date = None, _death_date = None, _species = None, _sex = None, _litter = None, _strain = None, _line = None, _description = None, _genotype = None):
        _data = dict(nickname = _nickname,
                    responsible_user = _responsible_user,
                    birth_date = _birth_date,
                    death_date = _death_date,
                    species = _species,
                    sex = _sex,
                    litter = _litter,
                    strain = _strain,
                    line = _line,
                    description = _description,
                    genotype = _genotype)

        print(_data)

        #requests.patch(ENDPOINT+'/'+_nickname,headers = self.apibase.headers, data= _data)



# !/usr/bin/python3
# -*- coding: utf-8 -*-

__version__ = "1.0.0"
__author__ = "Sérgio Copeto"
__credits__ = ["Sérgio Copeto", "Ricardo Ribeiro"]
__license__ = "Copyright (C) 2007 Free Software Foundation, Inc. <http://fsf.org/>"
__maintainer__ = ["Sérgio Copeto", "Ricardo Ribeiro"]
__email__ = ["ricardojvr@gmail.com"]
__status__ = "Development"

from confapp import conf

conf += 'pybpodgui_plugin_trial_timeline.settings'
#conf += 'pybpodgui_plugin_trial_timeline.resources'


import loggingbootstrap

# setup different loggers but output to single file
loggingbootstrap.create_double_logger("pybpodgui_plugin_trial_timeline", conf.APP_LOG_HANDLER_CONSOLE_LEVEL,
									  conf.APP_LOG_FILENAME,
									  conf.APP_LOG_HANDLER_FILE_LEVEL)


# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

logger = logging.getLogger(__name__)


class OutputChannel(object):
    """
    Available output channels
    These values must be set according to Bpod firmware specification.
    """

    #: LED
    LED = "LED"

    #: Valve1
    Valve1 = "Valve1"

    #: Valve2
    Valve2 = "Valve2"

    #: Valve3
    Valve3 = "Valve3"

    #: Valve4
    Valve4 = "Valve4"

    #: Serial 1
    Serial1 = "Serial1"

    #: Serial 2
    Serial2 = "Serial2"

    #: Serial 3
    Serial3 = "Serial3"

    #: Serial 4
    Serial4 = "Serial4"

    #: Serial 5
    Serial5 = "Serial5"

    #: SoftCode
    SoftCode = "SoftCode"

    #: BNC1
    BNC1 = "BNC1"

    #: BNC2
    BNC2 = "BNC2"

    #: PWM1
    PWM1 = "PWM1"

    #: PWM2
    PWM2 = "PWM2"

    #: PWM3
    PWM3 = "PWM3"

    #: PWM4
    PWM4 = "PWM4"

    #: GlobalTimerTrig
    GlobalTimerTrig = "GlobalTimerTrig"

    #: GlobalTimerCancel
    GlobalTimerCancel = "GlobalTimerCancel"

    #: GlobalCounterReset
    GlobalCounterReset = "GlobalCounterReset"



def generate_name(subjects, suffix):
    final_name = 'Untitled {suffix} {len}'.format(suffix=suffix, len=len(subjects))
    # rename current subject if the name already exists in the list
    if len(subjects) == 0:
        return final_name

    # check if there's any name that starts with the same name
    starts_with_name = list(filter(lambda x: x.startswith(final_name), subjects))
    starts_with_name_len = len(starts_with_name)
    if starts_with_name_len == 0:
        return final_name

    ends_with_paren = list(filter(lambda x: x.endswith(")"), starts_with_name))
    ends_with_paren_len = len(ends_with_paren)
    if starts_with_name_len != ends_with_paren_len:
        numbers = []
        for x in ends_with_paren:
            numbers.append(int(x[x.find('(') + len('('):x.rfind(')')]))
        if len(numbers) != 0:
            final_name = final_name + " ({0})".format(max(numbers) + 1)
        else:
            final_name = final_name + " ({0})".format(
                starts_with_name_len - (starts_with_name_len - ends_with_paren_len) + 1)
    return final_name



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

from confapp import conf


from AnyQt.QtWidgets import QApplication
from AnyQt.QtGui import QIcon
from AnyQt import QtCore

from pybpodgui_plugin.models.task.task_window import TaskWindow

logger = logging.getLogger(__name__)


class TaskTreeNode(TaskWindow):
    """
    Extends task window to show up in the project tree section.
    Define here actions related to the task tree node.

    **Properties**

        name
            Handles task tree node name.

        tree
            Returns project tree.

    **Methods**

    """

    def __init__(self, project):
        TaskWindow.__init__(self, project)

        self.create_treenode(self.tree)

    def create_treenode(self, tree):
        """
        Creates node for this task under the parent "Tasks" node.

        This methods is called when the task is first created.

        The following actions get assigned to node:
            * *Remove*: :meth:`TaskTreeNode.remove`.

        Sets key events:
            * :meth:`TaskTreeNode.node_key_pressed_event`


        :param tree: the project tree
        :type tree: pyforms.controls.ControlTree
        :return: new created node
        :return type: QtGui.QTreeWidgetItem
        """
        self.node = tree.create_child(self.name, self.project.tasks_node, icon=QIcon(conf.TASK_SMALL_ICON))
        self.node.key_pressed_event = self.node_key_pressed_event
        self.node.double_clicked_event = self.node_double_clicked_event
        self.node.window = self
        self.node.setExpanded(True)

        tree.add_popup_menu_option('Remove', self.remove, item=self.node, icon=QIcon(conf.REMOVE_SMALL_ICON))
        return self.node

    def node_key_pressed_event(self, event):
        """
        Sets key events for:
            * Remove task: :meth:`TaskTreeNode.remove`
            * Edit task code: :py:meth:`pybpodgui_plugin.models.task.task_dockwindow.TaskDockWindow.edit_btn_evt`

        :param event: key event
        """
        modifiers = QApplication.keyboardModifiers()

        if event.key() == QtCore.Qt.Key_O and modifiers == QtCore.Qt.ControlModifier:
            self.edit_btn_evt()
        elif event.key() in [QtCore.Qt.Key_Enter, QtCore.Qt.Key_Return]:
            self.edit_btn_evt()
        elif event.key() == QtCore.Qt.Key_Delete:
            self.remove()

    def node_double_clicked_event(self):
        """
        Fires event :py:meth:`pybpodgui_plugin.models.task.task_dockwindow.TaskDockWindow.edit_btn_evt` when tree node
        is double clicked.
        """
        self.edit_btn_evt()

    def remove(self):
        """

        Remove task from project and remove node from tree.

        .. seealso::
            * Task removal (dock window): :py:meth:`pybpodgui_plugin.models.task.task_dockwindow.TaskDockWindow.remove`.
            * Task removal (API): :meth:`pybpodgui_api.models.board.board_base.TaskBase.remove`.
            * Remove task from project: :meth:`pybpodgui_api.models.project.project_base.ProjectBase.__sub__`.

        """
        self.project -= self
        self.project.tasks_node.removeChild(self.node)

    @property
    def name(self):
        return TaskWindow.name.fget(self)

    @name.setter
    def name(self, value):
        TaskWindow.name.fset(self, value)
        if hasattr(self, 'node'):
            self.node.setText(0, value)
        if hasattr(self, '_code_editor'):
            self._code_editor.title = value
            self._code_editor.refresh_directory()

    @property
    def tree(self):
        return self.project.tree



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
import os
import uuid
from pybpodgui_api.models.setup.board_task import BoardTask
from pybpodgui_api.models.session import Session
from pybpodgui_api.models.subject import Subject
from pybpodgui_api.utils.generate_name import generate_name

logger = logging.getLogger(__name__)


class SetupBase(object):

    def __init__(self, experiment):
        """
        :ivar Experiment experiment: Experiment to which the Setup belongs to
        """
        self.uuid4 = uuid.uuid4()
        self.name = generate_name([x.name for x in experiment.setups], "setup")
        self.detached = False

        self.experiment = experiment
        self.board_task = self.create_board_task()

        self._sessions = []
        self._subjects = []
        self.board = None
        self.task = None

        self.experiment += self

    ##########################################################################
    ####### PROPERTIES #######################################################
    ##########################################################################

    @property
    def name(self):
        """
        Get and set setup name

        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    @property
    def subjects(self):
        """
        Get list of subjects

        :rtype: list(Subject)
        """
        return self._subjects

    @property
    def board(self):
        """
        Get and set setup board

        :rtype: Board
        """
        return self.board_task.board

    @board.setter
    def board(self, value):
        if isinstance(value, str):
            value = self.project.find_board(value)

        if self.board_task:
            self.board_task.board = value

    @property
    def task(self):
        """
        Get and set task

        :rtype: Task
        """
        return self.board_task.task

    @task.setter
    def task(self, value):
        if isinstance(value, str):
            value = self.project.find_task(value)

        if self.board_task:
            self.board_task.task = value

    @property
    def experiment(self):
        """
        Get and set the experiment

        :rtype: Experiment
        """
        return self._experiment

    @experiment.setter
    def experiment(self, value):
        self._experiment = value

    @property
    def project(self):
        """
        Get project

        :rtype: Project
        """
        return self.experiment.project

    @property
    def detached(self):
        return self._detached

    @detached.setter
    def detached(self, value):
        self._detached = value

    @property
    def sessions(self):
        """
        Get the list of sessions

        :rtype: list(Session)
        """
        return self._sessions

    @property
    def path(self):
        """
        Get and set setup path

        :rtype: str
        """
        if self.experiment.path is None:
            return None
        return os.path.join(self.experiment.path, 'setups', self.name)

    @property
    def last_session(self):
        """
        Get last created session

        :rtype: Session
        """
        try:
            order_sessions = sorted(self.sessions, key=lambda session: session.started)  # sort by end_date
            return order_sessions[-1]
        except IndexError:
            return None

    @property
    def net_port(self):
        if self.task and self.board:
            if self.task.trigger_softcodes:
                return self.board.net_port
            else:
                return None
        else:
            return None

    @property
    def update_variables(self):
        return self.board_task.update_variables

    ##########################################################################
    ####### FUNCTIONS ########################################################
    ##########################################################################

    def remove(self):
        """
        Remove the setup from the project
        """
        pass

    def create_board_task(self):
        """
        Create a new BoardTask object

        :rtype: BoardTask
        """
        return BoardTask(self)

    def create_session(self):
        """
        Create a new Session object

        :rtype: Session
        """
        return Session(self)

    def clear_subjects(self):
        # we need the copy because we are not allowed to change the same container that we are iterating
        subjects_copy = self.subjects.copy()
        for subj in subjects_copy:
            self -= subj

    def __unicode__(self):
        return self.name

    def __str__(self):
        return self.__unicode__()

    def __add__(self, obj):
        if isinstance(obj, Session) and obj not in self._sessions:
            self._sessions.append(obj)
        if isinstance(obj, Subject) and obj not in self._subjects:
            self._subjects.append(obj)
        if isinstance(obj, list):
            self._subjects.extend([x for x in obj if isinstance(x, Subject)])
        return self

    def __sub__(self, obj):
        if isinstance(obj, Session) and obj in self._sessions:
            self._sessions.remove(obj)
        if isinstance(obj, Subject) and obj in self._subjects:
            self._subjects.remove(obj)
        return self



# !/usr/bin/python3
# -*- coding: utf-8 -*-

__version__ = "1.0.0"
__author__ = "Ricardo Jorge Vieira Ribeiro"
__credits__ = ["Ricardo Ribeiro"]
__license__ = "Copyright (C) 2007 Free Software Foundation, Inc. <http://fsf.org/>"
__maintainer__ = ["Ricardo Ribeiro"]
__email__ = ["ricardo.ribeiro@research.fchampalimaud.org", "ricardojvr@gmail.com"]
__status__ = "Development"

import loggingbootstrap
from confapp import conf

conf += 'pybpodgui_plugin_stmdiagram.settings'

# setup different loggers but output to single file
loggingbootstrap.create_double_logger(
	"pybpodgui_plugin_stmdiagram", 
	conf.APP_LOG_HANDLER_CONSOLE_LEVEL,
	conf.APP_LOG_FILENAME,
	conf.APP_LOG_HANDLER_FILE_LEVEL
)

from .stmdiagram_window import StmDiagramWindow



#!/usr/bin/python
# -*- coding: utf-8 -*-

""" pyforms_gui.controls.ControlPlayer.ControlPlayer

"""
__author__ = "Ricardo Ribeiro"
__credits__ = ["Ricardo Ribeiro"]
__license__ = "MIT"
__version__ = "0.0"
__maintainer__ = "Ricardo Ribeiro"
__email__ = "ricardojvr@gmail.com"
__status__ = "Development"

import logging, platform, os, math
from .multiple_videocapture import MultipleVideoCapture

try:
    import cv2
except:
    raise Exception('OpenCV is not available. ControlPlayer will not be working')


from confapp 	 import conf
from AnyQt 			 import uic, _api
from AnyQt 			 import QtCore
from AnyQt.QtWidgets import QFrame	
from AnyQt.QtWidgets import QApplication
from AnyQt.QtWidgets import QMainWindow
from AnyQt.QtWidgets import QMessageBox

from pyforms_gui.controls.control_base import ControlBase

if _api.USED_API == _api.QT_API_PYQT5:
	import platform
	if platform.system() == 'Darwin':
		from pyforms_gui.controls.control_player.VideoQt5GLWidget import VideoQt5GLWidget as VideoGLWidget
	else:
		from pyforms_gui.controls.control_player.VideoGLWidget 	 import VideoGLWidget

elif _api.USED_API == _api.QT_API_PYQT4:	
	from pyforms_gui.controls.control_player.VideoGLWidget 		 import VideoGLWidget


logger = logging.getLogger(__name__)

class ControlPlayer(ControlBase, QFrame):

	def __init__(self, *args, **kwargs):
		self._video_widget = None  # GL widget

		QFrame.__init__(self)
		ControlBase.__init__(self, *args, **kwargs)

		self._multiple_files = kwargs.get('multiple_files', False)

		self._current_frame = None  # current frame image
		self._current_frame_index = None # current frame index

		self.process_frame_event = kwargs.get('process_frame_event', self.process_frame_event)
		
		self._speed = 1
		self.logger = logging.getLogger('pyforms')

		self._update_video_frame  = True # if true update the spinbox with the current frame
		self._update_video_slider = True  # if true update the slider with the current frame



	def init_form(self):
		# Get the current path of the file
		rootPath = os.path.dirname(__file__)

		# Load the UI for the self instance
		uic.loadUi(os.path.join(rootPath, "video.ui"), self)


		# Define the icon for the Play button
		self.videoPlay.setIcon(conf.PYFORMS_ICON_VIDEOPLAYER_PAUSE_PLAY)
		self.detach_btn.setIcon(conf.PYFORMS_ICON_VIDEOPLAYER_DETACH)

		self.detach_btn.clicked.connect(self.__detach_player_evt)

		self._video_widget = VideoGLWidget()
		self._video_widget._control = self
		self.videoLayout.addWidget(self._video_widget)
		self.videoPlay.clicked.connect(self.videoPlay_clicked)
		self.videoFrames.valueChanged.connect(self.video_frames_value_changed)
		self.videoProgress.valueChanged.connect(self.videoProgress_valueChanged)
		self.videoProgress.sliderReleased.connect(self.videoProgress_sliderReleased)
		self._timer = QtCore.QTimer(self)
		self._timer.timeout.connect(self.call_next_frame)

		self.form.horizontalSlider.valueChanged.connect(self.__rotateZ)
		self.form.verticalSlider.valueChanged.connect(self.__rotateX)

		self._current_frame = None
	
		self.view_in_3D = False

	##########################################################################
	############ FUNCTIONS ###################################################
	##########################################################################

	def play(self):
		try:
			self.videoPlay.setChecked(True)
			self._timer.start( 1000.0/float(self.fps+1) )
		except Exception as e:
			self.videoPlay.setChecked(False)
			logger.error(e, exc_info=True)

	def stop(self):
		self.videoPlay.setChecked(False)
		self._timer.stop()

	def hide(self): QFrame.hide(self)

	def show(self): QFrame.show(self)

	def refresh(self):
		if self._current_frame is not None:
			frame = self.process_frame_event(self._current_frame.copy())
			if isinstance(frame, list) or isinstance(frame, tuple):
				self._video_widget.paint(frame)
			else:
				self._video_widget.paint([frame])
		else:
			self._video_widget.paint(None)

	def save_form(self, data, path=None): return data

	def load_form(self, data, path=None): pass


	##########################################################################
	############ EVENTS ######################################################
	##########################################################################

	def process_frame_event(self, frame): return frame

	@property
	def double_click_event(self): return self._video_widget.onDoubleClick
	@double_click_event.setter
	def double_click_event(self, value): self._video_widget.onDoubleClick = value

	@property
	def click_event(self): return self._video_widget.onClick
	@click_event.setter
	def click_event(self, value):  self._video_widget.onClick = value

	@property
	def drag_event(self): return self._video_widget.onDrag
	@drag_event.setter
	def drag_event(self, value): self._video_widget.onDrag = value

	@property
	def end_drag_event(self): return self._video_widget.onEndDrag
	@end_drag_event.setter
	def end_drag_event(self, value): self._video_widget.onEndDrag = value

	@property
	def key_press_event(self):
		return self._video_widget.on_key_press
	@key_press_event.setter
	def key_press_event(self, value):
		self._video_widget.on_key_press = value

	@property
	def key_release_event(self): return self._video_widget.on_key_release
	@key_release_event.setter
	def key_release_event(self, value): self._video_widget.on_key_release = value

	##########################################################################
	############ PROPERTIES ##################################################
	##########################################################################

	@property
	def video_widget(self): return self._video_widget

	@property
	def next_frame_step(self): return self._speed
	@next_frame_step.setter
	def next_frame_step(self, value): self._speed = value

	@property
	def view_in_3D(self): return self._video_widget.onEndDrag
	@view_in_3D.setter
	def view_in_3D(self, value):
		self.form.horizontalSlider.setVisible(value)
		self.form.verticalSlider.setVisible(value)

	@property
	def video_index(self): return int(self._value.get(1)) if self._value else None

	@video_index.setter
	def video_index(self, value): self._value.set(1, value)

	@property
	def max(self):
		if self._value is None or self._value=='':
			return 0
		return int(self._value.get(7))

	@property
	def frame(self): return self._current_frame

	@frame.setter
	def frame(self, value):
		if isinstance(value, list) or isinstance(value, tuple):
			self._video_widget.paint(value)
		elif value is not None:
			self._video_widget.paint([value])
		else:
			self._video_widget.paint(None)
		QApplication.processEvents()

	@property
	def fps(self): 
		"""
			Return the video frames per second
		"""
		return self._value.get(5)

	@property
	def help_text(self): return self._video_widget._helpText

	@help_text.setter
	def help_text(self, value): self._video_widget._helpText = value

	@property
	def form(self): return self

	@property
	def frame_width(self): return self._value.get(3)

	@property
	def frame_height(self): return self._value.get(4)

	@property
	def is_playing(self): return self._timer.isActive()

	@property
	def value(self): return ControlBase.value.fget(self)

	@value.setter
	def value(self, value):
		self.form.setUpdatesEnabled(False)
		if value is None: 
			self.stop()
			self.videoControl.setEnabled(False)
			self.refresh()
		self._video_widget.reset()

		if value == 0:
			self._value = cv2.VideoCapture(0)
		elif isinstance(value, str) and value:

			open_multiplefiles = self._multiple_files

			if open_multiplefiles:
				open_multiplefiles = len(MultipleVideoCapture.search_files(value))>0

			if open_multiplefiles:
				msg = "Multiple files were found with the same name, do you wish to combine then in a single video?\n\n"
				for filepath in MultipleVideoCapture.search_files(value):
					msg += "- {filename}\n".format(filename=os.path.basename(filepath))

				reply = QMessageBox(
					QMessageBox.Question,
					'Open multiple files',
					msg,
					QMessageBox.No | QMessageBox.Yes
				).exec_()

				if reply == QMessageBox.Yes:
					open_multiplefiles = True
				else:
					open_multiplefiles = False

			if open_multiplefiles:
				self._value = MultipleVideoCapture(value)
			else:
				self._value = cv2.VideoCapture(value)
		else:
			self._value = value

		if self._value and value != 0:
			self.videoProgress.setMinimum(0)
			self.videoProgress.setValue(0)
			self.videoProgress.setMaximum(
				self._value.get(7))
			self.videoFrames.setMinimum(0)
			self.videoFrames.setValue(0)
			self.videoFrames.setMaximum(
				self._value.get(7))

		if self._value:
			self.videoControl.setEnabled(True)

		self.refresh()
		self.form.setUpdatesEnabled(True)


	##########################################################################
	############ PRIVATE FUNCTIONS ###########################################
	##########################################################################

	def __rotateX(self):
		self._video_widget.rotateX = self.form.verticalSlider.value()
		self.refresh()

	def __rotateZ(self):
		self._video_widget.rotateZ = self.form.horizontalSlider.value()
		self.refresh()

	

	

	def call_next_frame(self, update_slider=True, update_number=True, increment_frame=True):
		# move the player to the next frame
		self.form.setUpdatesEnabled(False)

		self._current_frame_index = self.video_index

		# if the player is not visible, stop
		if not self.visible: 
			self.stop()
			self.form.setUpdatesEnabled(True)
			return

		# if no video is selected
		if self.value is None: 
			self._current_frame = None
			self._current_frame_index = None
			return

		# read next frame
		(success, self._current_frame) = self.value.read()

		# increment frame index if the step is bigger than 1
		if increment_frame and self.next_frame_step > 1:
			self.video_index += self.next_frame_step

		# no frame available. leave the function
		if not success:
			self.stop()
			self.form.setUpdatesEnabled(True)
			return

		frame = self.process_frame_event(
			self._current_frame.copy()
		)

		# draw the frame
		if isinstance(frame, list) or isinstance(frame, tuple):
			self._video_widget.paint(frame)
		else:
			self._video_widget.paint([frame])

		if not self.videoProgress.isSliderDown():

			if update_slider and self._update_video_slider:
				self._update_video_slider = False
				self.videoProgress.setValue(self._current_frame_index)
				self._update_video_slider = True

			if update_number:
				self._update_video_frame = False
				self.videoFrames.setValue(self._current_frame_index)
				self._update_video_frame = True

		self.form.setUpdatesEnabled(True)


	def __detach_player_evt(self):
		"""
		Called by the detach button
		"""
		self._old_layout = self.parentWidget().layout()
		self._old_layout_index = self._old_layout.indexOf(self)
		self._detach_win = QMainWindow(parent=self.parent)
		self._detach_win.setWindowTitle('Player')
		self._detach_win.setCentralWidget(self)
		self.detach_btn.hide()
		self._detach_win.closeEvent = self.__detach_win_closed_evt
		self._detach_win.show()

	def __detach_win_closed_evt(self, event):
		"""
		Called when the detached window is closed
		"""
		self._old_layout.insertWidget(self._old_layout_index, self)
		self.detach_btn.show()
		self._detach_win.close()
		del self._detach_win

	def videoPlay_clicked(self):
		"""Slot for Play/Pause functionality."""
		if self.is_playing:
			self.stop()
		else:
			self.play()

	def convertFrameToTime(self, totalMilliseconds):
		# totalMilliseconds = totalMilliseconds*(1000.0/self._value.get(5))
		if math.isnan(totalMilliseconds): return 0, 0, 0
		totalseconds = int(totalMilliseconds / 1000)
		minutes = int(totalseconds / 60)
		seconds = totalseconds - (minutes * 60)
		milliseconds = totalMilliseconds - (totalseconds * 1000)
		return (minutes, seconds, milliseconds)

	def videoProgress_valueChanged(self):
		milli = self._value.get(0)
		(minutes, seconds, milliseconds) = self.convertFrameToTime(milli)
		self.videoTime.setText(
			"%02d:%02d:%03d" % (minutes, seconds, milliseconds))

	

	def videoProgress_sliderReleased(self):

		if not self.is_playing and self._update_video_slider:
			new_index = self.videoProgress.value()
			self._value.set(1, new_index)
			self.call_next_frame(update_slider=False, increment_frame=False)

	def video_frames_value_changed(self, pos):

		if not self.is_playing and self._update_video_frame:
			self._value.set(1, pos) # set the video position
			self.call_next_frame(update_number=False, increment_frame=False)

	def jump_forward(self):
		self.video_index += 20 * self.fps

	def jump_backward(self):
		self.video_index -= 20 * self.fps


#!/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
import re

import pyforms as app
from pyforms.basewidget import BaseWidget
from pyforms.controls import ControlList
from pyforms.controls import ControlCheckBox

from pybpodgui_plugin.models.setup.task_variable import TaskVariableWindow
from pybpodgui_api.models.setup.board_task import BoardTask

logger = logging.getLogger(__name__)


class BoardTaskWindow(BoardTask, BaseWidget):
    """
    Define here which fields from the board_task model should appear on the setup configuration window.

    The model fields shall be defined as UI components like text fields, buttons, combo boxes, etc.

    You may also assign actions to these components.

    .. seealso::
        This class heavy relies on the corresponding API module.

        :py:class:`pybpodgui_api.models.setup.board_task.BoardTask`

    **Properties**

        states
            A list of task states associated with this BoardTask. States are defined on the task code.

        events
            A list of task events associated with this BoardTask. Events are defined on the task code.

        variables
            A list of task variables associated with this BoardTask. Variables are defined on the task code.

    **Private attributes**

        _states
            :class:`pyforms.controls.ControlList`

            UI list to show BoardTask states.

        _events
            :class:`pyforms.controls.ControlList`

            UI list to show BoardTask events.

        _vars
            :class:`pyforms.controls.ControlList`

            UI list to show BoardTask variables.

        _sync_btn
            :class:`pyforms.controls.ControlButton`

            Button to sync variables with board. Pressing the button fires the event :meth:`BoardTaskWindow.sync_variables`.

        _load_btn
            :class:`pyforms.controls.ControlButton`

            Button to read task variables from board. Pressing the button fires the event :meth:`BoardTaskWindow._BoardTaskWindow__load_task_details`.

        _formset
            Describe window fields organization to PyForms.

    **Methods**

    """

    def __init__(self, setup):
        BaseWidget.__init__(self, "Variables config for {0}".format(setup.name))

        self._var_is_being_added = False

        self._updvars = ControlCheckBox('Update variables')
        self._vars = ControlList('Variables',
                                 add_function=self.__add_variable,
                                 remove_function=self.__remove_variable)

        BoardTask.__init__(self, setup)

        self._vars.horizontal_headers = ['NAME', 'TYPE', 'VALUE']
        self._vars.data_changed_event = self.__varslist_data_changed_evt

        self._formset = ['_updvars', '_vars']

        self._variable_rule = re.compile('^[A-Z0-9\_]+$')

    @property
    def update_variables(self):
        return self._updvars.value

    @update_variables.setter
    def update_variables(self, value):
        self._updvars.value = value

    def create_variable(self, name=None, value=None, datatype='string'):
        return TaskVariableWindow(self, name, value, datatype)

    def __varslist_data_changed_evt(self, row, col, item):

        # only verify if the list is being edited
        if self._var_is_being_added is True:
            return

        if col == 0 and item is not None:
            if not (self._variable_rule.match(item) and item.startswith('VAR_')):
                self.critical("The name of the variable should start with VAR_, should be alphanumeric and upper case.",
                              "Error")

                self._vars.set_value(
                    col, row,
                    'VAR_{0}'.format( self._vars.rows_count))

        elif col == 2:
            datatype_combo = self._vars.get_value(1, row)
            datatype = datatype_combo.value if datatype_combo else None
            if datatype == 'number' and isinstance(item, str) and not item.isnumeric():
                self.message("The value should be numeric.", "Error")
                self._vars.set_value(
                    col, row,
                    '0'
                )

    def __add_variable(self):
        self._var_is_being_added = True
        var = self.create_variable(
            'VAR_{0}'.format(self._vars.rows_count),
            '0'
        )
        self._var_is_being_added = False

    def __remove_variable(self):
        if self._vars.selected_row_index is not None:
            var = self.variables[self._vars.selected_row_index]
            self.variables.remove(var)
            self._vars -= -1

    def before_close(self):
        return False


# Execute the application
if __name__ == "__main__":
    app.start_app(BoardTaskWindow)



# !/usr/bin/python
# -*- coding: utf-8 -*-

import logging

from confapp import conf

from AnyQt.QtGui import QIcon
from pybpodgui_plugin_session_history.session_history import SessionHistory

logger = logging.getLogger(__name__)


class SessionTreeNode(object):
    def create_treenode(self, tree):
        """
        
        :param tree: 
        :return: 
        """
        node = super(SessionTreeNode, self).create_treenode(tree)

        self.sessionhistory_action = tree.add_popup_menu_option(
            'History', 
            self.open_sessionhistory_win,
            item=self.node,
            icon=QIcon(conf.SESSIONLOG_PLUGIN_ICON)
        )

        self.sessionhistory_detached_action = tree.add_popup_menu_option(
            'History (detached)', 
            self.open_sessionhistory_win_detached,
            item=self.node,
            icon=QIcon(conf.SESSIONLOG_PLUGIN_ICON)
        )

        return node

    def node_double_clicked_event(self):
        super(SessionTreeNode, self).node_double_clicked_event()

        
        self.open_sessionhistory_win()

    def open_sessionhistory_win(self):
        if self.is_running and self.setup.detached:
            return

        self.load_contents()

        #does not show the window if the detached window is visible
        if hasattr(self, 'sessionhistory_win_detached') and self.sessionhistory_win_detached.visible: return 

        if not hasattr(self, 'sessionhistory_win'):
            self.sessionhistory_win = SessionHistory(self)
            self.sessionhistory_win.show()
            self.sessionhistory_win.subwindow.resize(*conf.SESSIONLOG_PLUGIN_WINDOW_SIZE)
        else:
            self.sessionhistory_win.show()

        self.sessionhistory_action.setEnabled(False)
        self.sessionhistory_detached_action.setEnabled(False)

    def open_sessionhistory_win_detached(self):
        if self.is_running and self.setup.detached:
            return

        self.load_contents()
        
        #does not show the window if the attached window is visible
        if hasattr(self, 'sessionhistory_win') and self.sessionhistory_win.visible: return 

        if not hasattr(self, 'sessionhistory_win_detached'):
            self.sessionhistory_win_detached = SessionHistory(self)
            self.sessionhistory_win_detached.show(True)
            self.sessionhistory_win_detached.resize(*conf.SESSIONLOG_PLUGIN_WINDOW_SIZE)
        else:
            self.sessionhistory_win_detached.show(True)

        self.sessionhistory_action.setEnabled(False)
        self.sessionhistory_detached_action.setEnabled(False)



    def remove(self):
        if hasattr(self, 'sessionhistory_win'): self.mainwindow.mdi_area -= self.sessionhistory_win
        super(SessionTreeNode, self).remove()

    @property
    def name(self):
        return super(SessionTreeNode, self.__class__).name.fget(self)

    @name.setter
    def name(self, value):
        super(SessionTreeNode, self.__class__).name.fset(self, value)
        if hasattr(self, 'sessionhistory_win'): self.sessionhistory_win.title = value



from AnyQt.QtGui import QIcon
from confapp import conf
from pybpod_soundcard_module.module_gui import SoundCardModuleGUI


class ProjectsSoundCard(object):

    def register_on_main_menu(self, mainmenu):
        super(ProjectsSoundCard, self).register_on_main_menu(mainmenu)

        if len([m for m in mainmenu if 'Tools' in m.keys()]) == 0:
            mainmenu.append({'Tools': []})

        menu_index = 0
        for i, m in enumerate(mainmenu):
            if 'Tools' in m.keys():
                menu_index = i
                break

        mainmenu[menu_index]['Tools'].append('-')
        mainmenu[menu_index]['Tools'].append(
            {'Sound Card': self.open_soundcard_plugin, 'icon': QIcon(conf.SOUNDCARD_PLUGIN_ICON)})

    def open_soundcard_plugin(self):
        if not hasattr(self, 'soundcard_plugin'):
            self.soundcard_plugin = SoundCardModuleGUI(self)
            self.soundcard_plugin.show()
            self.soundcard_plugin.resize(*conf.SOUNDCARD_PLUGIN_WINDOW_SIZE)
        else:
            self.soundcard_plugin.show()

        return self.soundcard_plugin



__version__ 	= "0.1.4"
__author__ 		= ['Ricardo Ribeiro']
__credits__ 	= ["Ricardo Ribeiro"]
__license__ 	= "MIT"
__maintainer__ 	= ['Ricardo Ribeiro']
__email__ 		= ['ricardojvr@gmail.com']
__status__ 		= "Development"

from confapp import conf

conf += 'pybpod_rotaryencoder_module.settings'

from pybpod_rotaryencoder_module.module import RotaryEncoder as BpodModule


import os, shutil
from pathlib import Path
from sca.formats import json, csv
from sca import format_flags as flags
from sca.utils.send2trash_wrapper import send2trash


class Repository(json.scadict):

    def __init__(self, path, data=None, uuid4_id=None, def_url=None, def_text=None, software=None, fileformat='json', parent=None):
        json.scadict.__init__(self, data, uuid4_id, def_url, def_text, software)

        self.parent     = parent  # parent repository
        self.fileformat = fileformat
        self.name       = os.path.basename(path)
        self.path       = os.path.abspath(path)
        
        self.children   = []    # children repositories
        self.updaterepo = False

    def __add__(self, repo):
        """
        Add a repository recursively
        """
        repo_location = repo.path[len(self.path):]
        repos_names   = [x for x in repo_location.split(os.sep) if len(x.strip())>0]

        if len(repos_names)==0: return self

        current = self
        for repo_name in repos_names[:-1]:
            r = None

            for tmp in current.children:
                if tmp.name==repo_name:
                    r = tmp
                    break

            if r is None:
                r = Repository( os.path.join(current.path, repo_name), parent=current )
                current.children.append(r)
            current = r

        current.children.append(repo)
        return self


    @property
    def updaterepo(self):
        if self._updaterepo: return True

        for child in self.children:
            if child.updaterepo: return True
        return False
        
    @updaterepo.setter
    def updaterepo(self, value):
        self._updaterepo = value

        # when the value is false reset all the childrens ##
        if value==False:
            for c in self.children:
                c.updaterepo = False
        ####################################################

    
    @property
    def path(self):
        if self._path is None: return None
        return os.path.join(self.parent.path, self._path) if self.parent else self._path
        
    @path.setter
    def path(self, value):
        self._path = os.path.relpath(value, self.parent.path) if self.parent else value

    #######################################################################
    ### IO Functions ######################################################
    #######################################################################


    def open(self):
        """
        Open a repository and all its children
        """

        # check the fileformat of the repository, it can be json or csv
        #print(self.path, self.name)
        settings_file = os.path.join(self.path, self.name+'.json')  
        if os.path.isfile(settings_file):

            self.fileformat = 'json'
            # for json format
            with open(settings_file, 'r') as jsonfile:
                data = json.load(jsonfile)
                self.update(data, update_meta=True)
        else: 
            # for csv format
            settings_file = os.path.join(self.path, self.name+'.csv')
            if os.path.isfile(settings_file):
                self.fileformat = 'csv'

                with open(settings_file, 'r') as csvfile:

                    # load the file header to get the uid
                    csvreader = csv.reader(csvfile)

                    try:
                        next(csvreader)
                    except StopIteration:
                        pass

                    data = {}
                    
                    if csvreader.uuid4:          data.update({flags.UUID4_FLAG:           csvreader.uuid4})
                    if csvreader.created_on:     data.update({flags.CREATED_ON_FLAG:      csvreader.created_on})
                    if csvreader.updated_on:     data.update({flags.UPDATED_ON_FLAG:      csvreader.updated_on})
                    if csvreader.software:       data.update({flags.SOFTWARE_REF_FLAG:    csvreader.software})
                    if csvreader.def_text:       data.update({flags.DEFINITION_TEXT_FLAG: csvreader.def_text})
                    if csvreader.def_url:        data.update({flags.DEFINITION_URL_FLAG:  csvreader.def_url})
                    if csvreader.parents:        data.update({flags.PARENT_REF_FLAG:      csvreader.parents})
                    if csvreader.externals:      data.update({flags.EXTERNAL_REF_FLAG:    csvreader.externals})
                    if csvreader.external_files: data.update({flags.EXTERNAL_FILE_FLAG:   csvreader.external_files})
                    if csvreader.external_urls:  data.update({flags.DEFINITION_URL_FLAG:  csvreader.external_urls})
                   
                    self.update(data, update_meta=True)

        #load all the subrepositories
        for folder_name in sorted( os.listdir(self.path) ):
            folder_path = os.path.join(self.path, folder_name)
            if not os.path.isdir(folder_path): continue

            repo = self.find(folder_path)
            if repo is None:
                self += Repository(folder_path, parent=self).open()
        return self

    
    def commit(self):
        """
        Save the repository settings
        """
        self.updaterepo = True

        #for child in self.children: child.commit()
        
        # check if the repository was renamed and rename it if so #####
        renamed = self.name!=os.path.basename(self.path)
        # only rename the path if it was created
        if renamed: self.renameto(self.name)
        ###############################################################


        # create the folder if it does not exists ############################################
        if not os.path.exists(self.path): Path(self.path).mkdir(parents=True, exist_ok=True)
        #####################################################################################
        
        if self.fileformat=='json': self.dump2json(self.name+'.json', self)


        for child in self.children:
            if not child.updaterepo:
                shutil.rmtree(child.path) if os.path.isdir(child.path) else os.remove(child.path)
                self.children.remove(child)

        if self.parent is None: self.updaterepo = False

        return self.path


    def dump2json(self, jsonfile, data):
        """
        Dump to the repository a object to a jsonfile
        """
        jsonfile_path = os.path.join(self.path, jsonfile)
        with open(jsonfile_path, 'w') as jsonfile:
            json.dump(data, jsonfile)
        return self.path



    #######################################################################
    ### Utils Functions ###################################################
    #######################################################################

    def list(self):
        return sorted([v for v in self.children], key=lambda x:x.name )


    def sub_repository(self, *args, **kwargs):
        """
        Create a child repository recursively
        """
        uuid4       = kwargs.get('uuid4',       None )
        fileformat  = kwargs.get('fileformat', 'json')
        
        # if the uuid4 argument is defined, the function will look
        # to the previous repository in the hierarchy for an existing repository
        # and return it
        if uuid4:
            subpaths    = os.path.join(*args[:-1])
            parent_repo = self.find( subpaths )
            if parent_repo is not None:
                for repo in parent_repo.list():
                    if repo.uuid4==uuid4:
                        repo.name = args[-1]
                        return repo

        # if no uuid4 was passed in the argument or no repo was found
        # then we create a new repository
        subpaths = os.path.join(*args)
        repo     = Repository( os.path.join(self.path, subpaths), fileformat=fileformat, parent=self )
        repo.add_parent_ref(self.uuid4)
        self += repo

        return repo

    def find_uuid4(self, path, uuid4):
        repo = self.find(path)
        for r in self.list():
            if r.uuid4==uuid4: return r
        return None

    def find(self, path):
        """
        Find a repository by the path
        """
        repo_location = path
        repos_names   = [x for x in repo_location.split(os.sep) if len(x.strip())>0]

        if len(repos_names)==0: 
            return self
        elif len(repos_names)==1:
            base_location = repos_names[0]
            for c in self.children:
                if c.name==base_location: return c
            return None
        else:
            base_location = repos_names[0]
            if base_location in self.children:
                for c in self.children:
                    if c.name==base_location: 
                        repo = c
                        break
                return repo.find(os.sep.join(repos_names[1:]))
        
        return None


    def renameto(self, newname):

        # rename the repository file to the new name
        oldname        = os.path.basename(self.path)
        old_configfile = os.path.join(self.path, oldname+'.'+self.fileformat)
        new_configfile = os.path.join(self.path, newname+'.'+self.fileformat)
        
        # rename the repository folder to the new name
        newpath = os.path.join( os.path.dirname(self.path), newname)
        
        if os.path.exists(old_configfile):  shutil.move( old_configfile, new_configfile )
        shutil.move( self.path, newpath )

        self.path = newpath
        self.name = newname



    def pprint(self, prefix=''):
        print(prefix, self.name, self.path)
        for r in self.list():
            r.pprint(prefix+'  ')



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

from pybpodgui_plugin.models.board.windows.code_editor import CodeEditor
from pybpodgui_plugin.models.board.board_treenode import BoardTreeNode
from pybpodgui_plugin.models.board.windows.log_window import LogWindow

logger = logging.getLogger(__name__)


class BoardDockWindow(BoardTreeNode):
    """
    Dock window settings.
    Define here behaviors associated with board details section.

    **Properties**

        mainwindow
            Returns project main window.

    **Methods**

    """

    def __init__(self, project):
        super(BoardDockWindow, self).__init__(project)

        self._log_btn.value = self.open_log_window

    def show(self):
        """
        Select this window as the main window on the details section.
        """
        self.mainwindow.details.value = self
        super(BoardDockWindow, self).show()

    def focus_name(self):
        """
        Sets interface focus on the board name text field
        """
        self._name.form.lineEdit.setFocus()

    def remove(self):
        """

        Prompts user to confirm board removal and closes mdi windows associated with this board.

        .. seealso::
            This method extends board tree node :py:meth:`pybpodgui_plugin.models.board.board_treenode.BoardTreeNode.remove`.

        """
        reply = self.question('Board "{0}" will be deleted. Are you sure?'.format(self.name), 'Warning')

        if reply == 'yes':
            self.mainwindow.details.value = None
            if hasattr(self, '_code_editor'):
                self.mainwindow.mdi_area -= self._code_editor
            if hasattr(self, '_log'):
                self.mainwindow.mdi_area -= self._log

            super(BoardDockWindow, self).remove()

    def open_log_window(self):
        """
        Open board console window on the mdi section.
        """
        if not hasattr(self, '_log'):
            self._log = LogWindow(self)
        self.mainwindow.mdi_area += self._log

    def __edit_framework_evt(self):
        """
        Open code editor window on the mdi section for the framework source code.
        """
        if not self.path:
            self.message(self, "Cannot edit the file yet.", "You need to save the project first.")
        else:
            if not hasattr(self, '_code_editor'):
                self._code_editor = CodeEditor(self)

            self.mainwindow.mdi_area += self._code_editor

    @property
    def mainwindow(self):
        return self.project.mainwindow

    @property
    def name(self):
        return BoardTreeNode.name.fget(self)

    @name.setter
    def name(self, value):
        BoardTreeNode.name.fset(self, value)
        if hasattr(self, '_log'):
            self._log.title = value
        if hasattr(self, '_code_editor'):
            self._code_editor.title = value



# !/usr/bin/python3
# -*- coding: utf-8 -*-


from pybpodgui_api.exceptions.api_error import APIError


class InvalidSessionError(APIError):
    """ Exception raised when an invalid session is loaded"""

    def __init__(self, value, session_path=None, original_exception=None):
        APIError.__init__(self, value, original_exception)
        self.session_path = session_path



# !/usr/bin/python3
# -*- coding: utf-8 -*-
import logging, os

SETTINGS_PRIORITY = 999

from pyforms_gui.utils.plugins_finder import PluginsFinder

GENERIC_EDITOR_MODEL = "pyforms_generic_editor.models.projects.Projects"
GENERIC_EDITOR_TITLE = "Set the variable GENERIC_EDITOR_TITLE in your settings file to change the title"

GENERIC_EDITOR_PLUGINS_PATH = None
GENERIC_EDITOR_PLUGINS_LIST = []

GENERIC_EDITOR_WINDOW_GEOMETRY = 100, 100, 1200, 800

GENERIC_EDITOR_PLUGINS_FINDER = PluginsFinder()

PYFORMS_STYLESHEET = ''
PYFORMS_STYLESHEET_DARWIN = ''

APP_LOG_FILENAME = 'app.log'
APP_LOG_HANDLER_FILE_LEVEL = logging.DEBUG
APP_LOG_HANDLER_CONSOLE_LEVEL = logging.INFO

PYFORMS_LOG_HANDLER_FILE_LEVEL = logging.DEBUG
PYFORMS_LOG_HANDLER_CONSOLE_LEVEL = logging.INFO

DEFAULT_PROJECT_PATH = None



# !/usr/bin/python3
# -*- coding: utf-8 -*-

from confapp import conf

from AnyQt.QtGui import QIcon

from pybpodgui_api.models.setup import Setup
from pybpodgui_plugin.models.session.session_signals import SessionSignals


class SessionUIBusy(SessionSignals):
    """

    """

    def update_ui(self):
        """

        """
        if not hasattr(self, 'node'):
            return

        if self.setup.status in [
            Setup.STATUS_RUNNING_TASK
        ]:
            self.node.setIcon(0, QIcon(conf.PLAY_SMALL_ICON))
            for s in self.subjects_nodes.values():
                s.setIcon(0, QIcon(conf.PLAY_SMALL_ICON))
            self.running = True
        else:
            self.node.setIcon(0, QIcon())
            for s in self.subjects_nodes.values():
                s.setIcon(0, QIcon())
            self.running = False



# !/usr/bin/python3
# -*- coding: utf-8 -*-

from pybpodapi.com.messaging.base_message import BaseMessage


class StdoutMessage(BaseMessage):
    """
    Print statement on bpod protocol

    .. seealso::

        :py:class:`pybpodgui_plugin.com.messaging.base_message.BoardMessage`

    """

    MESSAGE_TYPE_ALIAS = "stdout"
    MESSAGE_COLOR = (150, 150, 150)
